program molsoundplayer;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  IniFiles,
  SDL,
  SDL_Mixer,
  Libc,
  usharedmem in 'usharedmem.pas';
  
Const
  VERSION = 'V0.1';
    
Var
  GeneralSounds : Array  of PMix_Chunk;
  AmbientSounds : Array of PMix_Chunk;
  AmbientSoundNo: Integer;
  
function ReadSounds : Boolean;
// read the molsounds.ini and loads the sound files into mem, ready for
// playing
var
  IniSounds : tIniFile;
  iLoop     : Integer;
  iTemp     : Integer;
  sTemp     : String;

  procedure LoadSound (soundname : String);
  Begin
    sTemp := IniSounds.ReadString ('General',soundname, '');
    iTemp := IniSounds.ReadInteger ('General', soundname+'num', -1);
    if (sTemp <> '') and (iTemp <> -1) Then
      Generalsounds[iTemp] := Mix_LoadWAV( PChar (sTemp));
  End;

  procedure LoadAmbientSound (SoundNo : Integer);
  Begin
    sTemp := IniSounds.ReadString ('AmbientSounds','sound' + IntToStr(SoundNo), '');
    if (sTemp <> '') Then
      AmbientSounds[SoundNo-1] := Mix_LoadWAV( PChar (sTemp));
  End;
    
Begin
  Result := TRUE;  // default to this for now
  
  IniSounds := tIniFile.Create ('sounds.ini');
  iTemp := IniSounds.ReadInteger('General', 'numsounds', 0);
  SetLength (GeneralSounds, iTemp);

  // initialise the sounds array
  for iLoop := 0 to Length (GeneralSounds)-1 do
    Generalsounds[iLoop] := NIL;
    
  if iTemp <> 0 Then
  Begin
    LoadSound ('getcircuit');
    LoadSound ('getcoin');
    LoadSound ('completed');
    LoadSound ('clocksound1');
    LoadSound ('clocksound2');
    LoadSound ('walk');
    LoadSound ('cantmove');
    LoadSound ('buy');
    LoadSound ('bomb');
    LoadSound ('death');
  End;  

  AmbientSoundNo := IniSounds.ReadInteger('AmbientSounds', 'ambientnum', 10);
  iTemp := IniSounds.ReadInteger('AmbientSounds','numsounds', 0);
  SetLength (AmbientSounds, iTemp);
  for iLoop := 0 to Length (AmbientSounds)-1 do
    AmbientSounds[iLoop] := NIL;
  
  For iLoop := 1 to iTemp do
    LoadAmbientSound (iLoop);
    
  IniSounds.Free;  
End;    
    
function MolmanProcessAlive : Boolean;
// checks to see if molecule man is still running. 
var
  iSize : Integer;
  f     : File;
  
begin
  Libc.system('ps h -C molman > /tmp/molsound.tmp' );
  AssignFile (f, '/tmp/molsound.tmp');
  Reset (f,1);
  iSize := FileSize (f);
  CloseFile (f);

  // if molecule man is still running, iSize will be > 0
  Result := iSize <> 0;
end;  

var
  iLoop : Integer;
  ptrSounds : pSoundRec;
  iOldMagicNum : Integer;

begin
 Writeln ('Molecule Man Sound Player ',VERSION);
 
 Randomize;
 
 ptrSounds := GetSharedMem;
 if Integer (ptrSounds)=  -1 Then
 Begin
   writeln ('Failed to map shared mem', Libc.Errno);
   DeleteSharedMem;
   exit;
 End;
 
 if (SDL_Init(SDL_INIT_AUDIO ) < 0) then
 Begin
   writeln ('SDL init failed');
   exit;
 end;

 if (Mix_OpenAudio(22050, AUDIO_U8, 1, 512) < 0) then
  begin
    writeln ('Couldn''t set 22050 Hz 8-bit audio - : ');
    exit;
  end;

  ReadSounds;
  
  iOldMagicNum := ptrSounds^.MagicNo;
  ptrSounds^.SoundInfo[1].SoundNo:=0;
  While (MolmanProcessAlive) do
  Begin
    if ptrSounds^.MagicNo <> iOldMagicNum Then
    Begin
      iOldMagicNum := ptrSounds^.MagicNo;
      
      // a sound needs to be played
      For iLoop := 1 to MAX_NUM_SOUNDS do
        if ptrSounds^.SoundInfo[iLoop].SoundNo <> NO_SOUND Then
          if ptrSounds^.SoundInfo[iLoop].SoundNo <> AmbientSoundNo Then
            Mix_PlayChannel (ptrSounds^.SoundInfo[iLoop].SoundChn,
                          GeneralSounds[ptrSounds^.SoundInfo[iLoop].SoundNo],
                          0)
          else
            Mix_PlayChannel (ptrSounds^.SoundInfo[iLoop].SoundChn,
                          AmbientSounds [Random (Length (AmbientSounds))],
                          0)
               
    End;   

    sleep (10);  

  End;
  
  writeln ('quitting');
  For iLoop := 0 to Length (GeneralSounds) -1 do
    if Assigned (GeneralSounds[iLoop]) Then
      Mix_FreeChunk(Generalsounds[iLoop]);

  For iLoop := 0 to Length (AmbientSounds) -1 do
    if Assigned (AmbientSounds[iLoop]) Then
      Mix_FreeChunk(AmbientSounds[iLoop]);
                        
  Mix_CloseAudio;

  if Assigned (ptrSounds) Then
  begin
    DetatchSharedMem (ptrSounds);
    DeleteSharedMem;
  End;
end.
 




 