
unit uDesigner;

interface

uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, QImgList, QActnList, QMenus, QComCtrls, QGrids, uMaze,
  QExtCtrls, QButtons;

const 
  VERSION = 'V0.91';

type
  TfrmDesigner = class(TForm)
    lboMazeObjects: TListBox;
    ToolBar1: TToolBar;
    MainMenu1: TMainMenu;
    file1: TMenuItem;
    New1: TMenuItem;
    ActionList1: TActionList;
    actNew: TAction;
    actOpen: TAction;
    actSave: TAction;
    actExit: TAction;
    Open1: TMenuItem;
    Save1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    DrawGrid1: TDrawGrid;
    Label1: TLabel;
    Bevel1: TBevel;
    imgCur: TImage;
    SaveDialog1: TSaveDialog;
    OpenDialog1: TOpenDialog;
    Options1: TMenuItem;
    Autoset1: TMenuItem;
    N2: TMenuItem;
    ViewMaze1: TMenuItem;
    StatusBar1: TStatusBar;
    actAutoSet: TAction;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    actViewMaze: TAction;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    imgActions: TImageList;
    btnScrollRight: TSpeedButton;
    lblXPos: TLabel;
    btnScrollLeft: TSpeedButton;
    lblYPos: TLabel;
    btnScrollUp: TSpeedButton;
    btnScrollDown: TSpeedButton;
    actAboutDesigner: TAction;
    ToolButton10: TToolButton;
    ToolButton11: TToolButton;
    Help1: TMenuItem;
    About1: TMenuItem;
    actInstructions: TAction;
    ToolButton12: TToolButton;
    N3: TMenuItem;
    Instructions1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure actNewExecute(Sender: TObject);
    procedure actOpenExecute(Sender: TObject);
    procedure actExitExecute(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure lboMazeObjectsDrawItem(Sender: TObject; Index: Integer;
      Rect: TRect; State: TOwnerDrawState; var Handled: Boolean);
    procedure DrawGrid1DrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure lboMazeObjectsClick(Sender: TObject);
    procedure DrawGrid1Click(Sender: TObject);
    procedure actSaveExecute(Sender: TObject);
    procedure actAutoSetExecute(Sender: TObject);
    procedure DrawGrid1DblClick(Sender: TObject);
    procedure DrawGrid1KeyPress(Sender: TObject; var Key: Char);
    procedure actViewMazeExecute(Sender: TObject);
    procedure btnScrollRightClick(Sender: TObject);
    procedure btnScrollLeftClick(Sender: TObject);
    procedure btnScrollUpClick(Sender: TObject);
    procedure btnScrollDownClick(Sender: TObject);
    procedure DrawGrid1TopLeftChanged(Sender: TObject);
    procedure DrawGrid1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure actAboutDesignerExecute(Sender: TObject);
    procedure actInstructionsExecute(Sender: TObject);
    procedure DrawGrid1MouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure DrawGrid1MouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
  private
    { Private declarations }
    fMolMaze : TMolMaze;
    fCurObject : Byte;
    fMazeFileName : String;
    fAutoSet      : Boolean;
    fNumGraphics  : Integer;
    fScrolling    : Boolean;
    fNumCircuits  : Integer;
    
    // vars used for keeping track of which location is being views
    fMaxXpos      : Integer;
    fMaxYPos      : Integer;
    fXPos         : Integer;
    fYPos         : Integer;
    fKey          : Word;

    procedure ScrollHorizontally;
    procedure ScrollVertically;
    procedure ResetLocPositions;
    procedure UpdatePosLabels;
    procedure UpdateStatusPanel;
  public
    { Public declarations }
  end;

var
  frmDesigner: TfrmDesigner;

implementation

uses
  IniFiles, umazeviewer, umolgraphics, uAboutDesigner, uDesignerInstructions,
  uCompat;
  
{$R *.xfm}

Const
  PNL_AUTO=2;
  PNL_MOL_LOC=0;
  PNL_CIRCUITS=1;
  SCROLL_DELAY_TIME = 40;
  CURSOR_KEY_UP = 4115;
  CURSOR_KEY_DOWN = 4117;
  CURSOR_KEY_LEFT = 4114;
  CURSOR_KEY_RIGHT = 4116;
  
procedure TfrmDesigner.FormCreate(Sender: TObject);
var
  IniGraph     : TIniFile;
  iNumGRaphics : Integer;
  iLoop        : Integer;
  tmpBitMap    : TBitMap;
  sFilename    : String;
  sSection     : String;
  fError       : Boolean;
  
begin

  // read the graphics data for each of the maze objects, and add it to
  // the listbox
  DrawGRid1.ColCount := MAZE_SIZE;
  DrawGrid1.RowCount := MAZE_SIZE;

  IniGraph := TIniFile.Create (RespecFileName(ExtractFilePath
                               (Application.ExeName) +'/molgraphics.ini'));
  iNumGraphics := IniGraph.ReadInteger('General','numgraphics', 0);
  fNumGraphics := iNumGraphics;

  fError := FALSE;

  For iLoop := 0 to iNumGraphics do
  Begin
    tmpBitMap := tBItmap.Create;

    sSection := 'MolGraphic'+IntToStr (iLoop);
    if IniGraph.ReadBool (sSection, 'moleculeman', FALSE) Then
      sFilename := IniGraph.ReadString (sSection, 'downright1','')
    else
      if IniGraph.ReadBool (sSection, 'animated', FALSE) Then
        sFileName := IniGraph.ReadString (sSection, 'graphic1','')
      else
        sFilename := IniGraph.ReadString (sSection, 'graphic', '');

    try
      sFileName := ExtractFilePath (Application.ExeName)+sFileName;
      tmpBitmap.LoadFromFile (RespecFileName (sFileName));
      lboMazeObjects.Items.Add (IntToStr (iLoop));
      lboMazeObjects.Items.Objects [lboMazeObjects.Items.Count-1] := tmpBitMap;
    except
      MessageDlg ('Fatal Error', 'Could not load graphics file '+sFilename +#13#10+
                  'Designer will now close.', mtError, [mbOk], 0);
      fError := TRUE;
      Break;
    end;
  End;
      
  IniGraph.Free;

  if fError Then
    Halt;
    
  // set up a default, empty maze
  fMolMaze := TMolMaze.Create;
  fMolMaze.ClearMaze;

  // set defaults for everything else that needs it
  ResetLocPositions;
  fMazeFileName := '';
  fAutoSet := FALSE;
  UpdateStatusPanel;
  fScrolling := FALSE;
end;

procedure TfrmDesigner.actNewExecute(Sender: TObject);
begin
  If MessageDlg ('This will clear the existing maze. Are you sure ?',
                  mtWarning, [mbYes, mbNo], 0) = mrNo Then 
     exit;
                  
  // clear out the maze                
  fMolMaze.ClearMaze;
  fMazeFileName := '';
  ResetLocPositions;

  // ensure the grid is updated
  DrawGrid1.Repaint;
end;

procedure TfrmDesigner.actOpenExecute(Sender: TObject);
begin
 if fMazeFileName <> '' Then
    OpenDialog1.FileName := fMazeFilename;
  if OpenDialog1.Execute Then
    if fMolMaze.LoadMaze (OpenDialog1.Filename) Then
    Begin
      fMazeFilename := OpenDialog1.Filename;
      fNumCircuits := fMolMaze.TotalNumCircuits;
      ResetLocPositions;
      DrawGrid1.Repaint;
      UpDateStatusPanel;
    end
    else
      MessageDlg ('Error', 'Could not read maze from ' + fMazeFilename,
                  mtError, [mbOk], 0);
  
end;

procedure TfrmDesigner.actExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfrmDesigner.FormDestroy(Sender: TObject);
var
  iLoop : Integer;

begin
  // free memory used by the bitmaps in the listbox
  for iLoop := 0 to lboMazeObjects.Items.Count -1 do
    With lboMazeObjects.Items.Objects[iLoop] As TBitMap do
      Free;
end;

procedure TfrmDesigner.lboMazeObjectsDrawItem(Sender: TObject; Index: Integer;
  Rect: TRect; State: TOwnerDrawState; var Handled: Boolean);
  
var
  BitMap : TBitMap;
  
begin
   // draw the bitmap of the listbox item specified by Index
   With (Sender As TListBox).Canvas do
   Begin
     FillRect (Rect);
     BitMap := (Sender As TListBox).Items.Objects[Index] As TBitMap;
     Draw (Rect.Left+1,  Rect.Top, Bitmap);
   End;
end;

procedure TfrmDesigner.DrawGrid1DrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  bTemp  : Byte;
  Bitmap : TBitmap;  
  
begin
  // draws a maze object in the grid ...
  
  With (Sender As TDrawGRid).Canvas do
  Begin
    // clear out whatever's there to begin with
    FillRect (Rect);

    bTemp := fMolMaze.GetMazeObject(ACol, ARow);
    // specify a safe default if the graphic specified in the maze isn't
    // in the listbox
    
    if bTemp > fNumGraphics Then
      bTemp := 5;

    if bTemp <> 0 Then
    Begin
      Bitmap := (lboMazeObjects.Items.Objects[bTemp]) As TBitmap;
      Draw (Rect.Left +1, Rect.Top+1, Bitmap);
    End;
  End;
  
end;

procedure TfrmDesigner.lboMazeObjectsClick(Sender: TObject);
var
  BitMap : TBitmap;
  Rect   : TRect;
  
begin
  // draw the graphic clicked on in the listbox in the current selection box
  Rect.Top := 0;
  Rect.Left := 0;
  Rect.Right := imgCur.Width;
  Rect.Bottom := imgCur.Height;
  BitMap := (Sender As TListBox).Items.Objects[(Sender As TListBox).ItemIndex] As TBitMap;
  fCurObject := (Sender As TListBox).ItemIndex;
  
  With imgCur.Canvas do
  Begin
    FillRect (Rect);
    Draw (Rect.Left +1, Rect.Top,  Bitmap);
  End;
end;

procedure TfrmDesigner.DrawGrid1Click(Sender: TObject);
  
begin
  // if autoset is on, call the doubleclick event to set the
  // maze item to the currently selected object
  if not fAutoSet Then
    Exit;

  DrawGrid1DblClick (Sender);  
end;

procedure TfrmDesigner.actSaveExecute(Sender: TObject);
begin
 if fMazeFileName <> '' Then
    SaveDialog1.FileName := fMazeFilename;
  if SaveDialog1.Execute Then
  Begin
    if FileExists (SaveDialog1.Filename) then if
      MessageDlg ('Warning', SaveDialog1.Filename +' already exists. Overwrite ?',
                  mtWarning, [mbYes, mbNo], 0) = mrNo Then 
           exit;
      
    if fMolMaze.SaveMaze (SaveDialog1.Filename) Then
      fMazeFilename := SaveDialog1.Filename
    else
      MessageDlg ('Error', 'Could not save maze to ' + fMazeFilename,
                  mtError, [mbOk], 0);
  End;                
end;

procedure TFrmDesigner.UpdateStatusPanel;
begin
  if FAutoSet Then
    StatusBar1.Panels[PNL_AUTO].Text := 'Auto set = ON'
  else
    StatusBar1.Panels[PNL_AUTO].Text := 'Auto set = OFF';

  StatusBar1.Panels[PNL_MOL_LOC].Text := 'Molecule Man location : x= '+
                                IntToStr (FMolMaze.MolManXPos div (MOL_LOCATION_SIZE-1)) + ' y= '+
                                IntToStr (FMolMaze.MolManYpos div (MOL_LOCATION_SIZE-1));  
  StatusBar1.Panels[PNL_CIRCUITS].Text := 'Number of circuits : '+IntToStr (fNumCircuits);                              
end;

procedure TfrmDesigner.actAutoSetExecute(Sender: TObject);
begin
  FAutoSet := Not FAutoSet;
  actAutoSet.Checked := FAutoSet;
  UpdateStatusPanel;
end;

procedure TfrmDesigner.DrawGrid1DblClick(Sender: TObject);
var
  Rect : TRect;
  bRecountCircuits : Boolean;
  
begin

  // if we're adding or removing circuits we need to recalculate the total;
  bRecountCircuits := (fCurObject = fMolMaze.CircuitGraphicNo) or
                      (FMolMaze.GetMazeObject(DrawGrid1.Col, DrawGrid1.Row) = fMolMaze.CircuitGraphicNo);

  // erase molecule man from his old position if necessary
  if fCurObject = fMolMaze.MolGRaphicNo Then
    fMolMaze.SetMazeObject (fMolMaze.MolManXPos, fMolMaze.MolManYPos, 0);
                        
  FMolMaze.SetMazeObject(DrawGrid1.Col, DrawGrid1.Row, fCurObject);

  
  if fCurObject = fMolMaze.MolGRaphicNo Then 
  Begin
    fMolMaze.GetMolManLocation;
    UpdateStatusPanel;
    // redraw the whole grid to erase the old molecule man
    DrawGrid1.Repaint;
  End
  else
  Begin
    // otherwise, just repaint the current cell
    Rect := DrawGRid1.CellRect(DrawGrid1.Col, DrawGrid1.Row);
    DrawGrid1.InvalidateRect(Rect, TRUE);
  End;
  
  if bRecountCircuits Then
  Begin
    fNumCircuits := fMolMaze.TotalNumCircuits;
    UpdateStatusPanel;
  End;
                                                   
end;

procedure TfrmDesigner.DrawGrid1KeyPress(Sender: TObject; var Key: Char);
begin
  // a press of the space key changes an object in the maze to the 
  // current selection
  if key= ' ' THen
    DrawGrid1DblClick (Sender);
end;

procedure TfrmDesigner.actViewMazeExecute(Sender: TObject);
var
FrmMazeViewer : TFrmMazeViewer;
begin
  FrmMazeViewer := TFrmMazeViewer.Create(Self);
  FrmMazeViewer.MolMaze := fMolMaze;
  FrmMazeViewer.GraphicList := lboMazeobjects.Items;
  FrmMazeViewer.DrawMaze;
  FrmMAzeViewer.ShowModal;
  FrmMazeViewer.Free;
end;

procedure TfrmDesigner.ScrollHorizontally;
var
  iLoop    : Integer;
  iOldCol  : Integer;  
  iNewCol  : Integer;
  
begin

  fScrolling := TRUE;

  // work out where we're scrolling from and to
  iOldCol:= DrawGrid1.LeftCol;
  iNewCol := fXPos* (MOL_LOCATION_SIZE-1);
  
  if iNewCol > iOldCol Then
    for iLoop := iOldCol to iNewCol do
    Begin
      // scrolling right - one column at a time until the next location
      // is fully in view
      DrawGrid1.LeftCol := iLoop;

      // let the display update
      Application.ProcessMessages;

      // small delay so that the user can see the scrolling effect
      Sleep (SCROLL_DELAY_TIME);
    End;

  if iNewCol < iOldCol Then
    for iLoop := iOldCol Downto iNewCol do
    Begin
      // scrolling left
      DrawGrid1.LeftCol := iLoop;
      Application.ProcessMessages;
      Sleep (SCROLL_DELAY_TIME);
    End;

  fScrolling := FALSE;
    
end;

procedure TfrmDesigner.ScrollVertically;
var
  iLoop    : Integer;
  iOldRow  : Integer;  
  iNewRow  : Integer;

begin

 fScrolling := TRUE;
 
 iOldRow:= DrawGrid1.TopRow;
 iNewRow := fYPos* (MOL_LOCATION_SIZE-1);
  
  if iNewRow > iOldRow Then
    for iLoop := iOldRow to iNewRow do
    Begin
      DrawGrid1.TopRow := iLoop;
      Application.ProcessMessages;
      Sleep (SCROLL_DELAY_TIME);
    End;

  if iNewRow < iOldRow Then
    for iLoop := iOldRow Downto iNewRow do
    Begin
      DrawGrid1.TopRow := iLoop;
      Application.ProcessMessages;
      Sleep (SCROLL_DELAY_TIME);
    End;
   fScrolling := FALSE;  
end;


procedure TFrmDesigner.UpdatePosLabels;
Begin
  lblXPos.Caption := 'X='+IntToStr (fXPos);
  lblYPos.Caption := 'Y='+IntToStr (fyPos);
End;

procedure TFrmDesigner.ResetLocPositions;
Begin
  fMaxXpos := MAZE_SIZE_IN_LOCS-1;
  fMaxYpos := MAZE_SIZE_IN_LOCS-1;
  fXpos := 0;
  fYPos := 0;
  UpdatePosLabels;
  fScrolling := TRUE;
  DrawGrid1.LeftCol := 0;
  DrawGrid1.TopRow := 0;
  fScrolling := FALSE;
End;

procedure TfrmDesigner.btnScrollRightClick(Sender: TObject);
begin
  if fXpos <> fMaxXPos Then
  Begin
    inc (fXPos);
    UpdatePosLabels;
    btnScrollRight.OnClick := NIL;
    ScrollHorizontally;
    btnScrollRight.OnClick := btnScrollRightClick;
  End;
end;

procedure TfrmDesigner.btnScrollLeftClick(Sender: TObject);
begin
  if fXpos <> 0 Then
  Begin
    dec (fXPos);
    btnScrollLeft.OnClick := NIL;
    UpdatePosLabels;
    ScrollHorizontally;
    btnScrollLeft.OnClick := btnScrollRightClick;
  End; 
end;

procedure TfrmDesigner.btnScrollUpClick(Sender: TObject);
begin
  if fYpos <> 0 Then
  Begin
    btnScrollUp.OnClick := NIL;
    dec (fYPos);
    UpdatePosLabels;
    ScrollVertically;
    btnScrollUp.OnClick := btnScrollUpClick;
  End;
end;

procedure TfrmDesigner.btnScrollDownClick(Sender: TObject);
begin
  if fYpos <> fMaxYPos Then
  Begin
    btnScrollDown.OnClick := NIL;
    inc (fYPos);
    UpdatePosLabels;
    ScrollVertically;
    btnScrollDown.OnClick := btnScrollDownClick;
  End;
end;

procedure TfrmDesigner.DrawGrid1TopLeftChanged(Sender: TObject);
var
  iTemp : Integer;
  
begin
  // if this event has been triggered by a click of one of the scroll buttons
  // and not by the user navigating around the grid, we need to exit
  if fScrolling Then Exit;

  if fKey = CURSOR_KEY_RIGHT Then
  Begin
    // see if the user has navigated onto the next location, and if so, scroll
    // to bring it fully into view
    iTemp := (DrawGrid1.LeftCol+MOL_LOCATION_SIZE-1) div (MOL_LOCATION_SIZE-1) ;
    if iTemp > fXPos Then 
      btnScrollRightClick (NIL);
    Exit;
  End;

  if fKey = CURSOR_KEY_LEFT Then
  Begin
    iTemp := DrawGrid1.LeftCol div (MOL_LOCATION_SIZE-1);
    if iTemp < fXPos Then 
      btnScrollLeftClick (NIL);
      Exit;
  End;

  if fKey = CURSOR_KEY_DOWN Then
  Begin
    iTemp := (DrawGrid1.TopRow+MOL_LOCATION_SIZE-1) div (MOL_LOCATION_SIZE-1) ;
    if iTemp > fYPos Then 
      btnScrollDownClick (NIL);
    Exit;
  End;

  if fKey = CURSOR_KEY_UP Then
  Begin
    iTemp := DrawGrid1.TopRow div (MOL_LOCATION_SIZE-1);
    if iTemp < fYPos Then 
      btnScrollUpClick (NIL);
  End;
end;

procedure TfrmDesigner.DrawGrid1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  fKey := Key;
end;

procedure TfrmDesigner.actAboutDesignerExecute(Sender: TObject);
var
  frmAboutDesigner : TFrmAboutDesigner;
  
begin
  frmAboutDesigner := TFrmAboutDesigner.Create (Self);
  frmAboutDesigner.SetVersionNo(VERSION);
  frmAboutDesigner.ShowModal;
  frmAboutDesigner.Free;
end;

procedure TfrmDesigner.actInstructionsExecute(Sender: TObject);
var
  frmDesignerInstructions : TFrmDesignerInstructions;
  
begin
  frmDesignerInstructions := TFrmDesignerInstructions.Create (Self);
  frmDesignerInstructions.ShowModal;
  frmDesignerInstructions.Free;
end;

procedure TfrmDesigner.DrawGrid1MouseWheelDown(Sender: TObject;
  Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
  fKey := CURSOR_KEY_DOWN;
end;

procedure TfrmDesigner.DrawGrid1MouseWheelUp(Sender: TObject;
  Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
  fKey := CURSOR_KEY_UP;
end;

end.
