{
  Base class for graphic objects in Molecule Man.

  Don't instantiate objects of this type. You must always derive
  objects from this class and use them instead}

  
unit uGraphic;

interface

uses
  QGraphics, Types;
  
type
  tGraphicObject = class
    private
      fCanvas :    TCanvas; // surface object is painted on
      fxWorld :    Integer; //
      fyWorld :    Integer; // world coordinates
      fzWorld :    Integer; //
      fwdthWorld : Integer; // width in world coordinates
      fhghtWorld : Integer; // height in world coordinates 
      fGraphic   : TBitmap; // graphic object;
      
      fClearGraphics: Boolean; // specifies whether or not this object
                               // created it's bitmap, and therefore
                               // whether or not it needs to free it
    public
      constructor Create;
      destructor  Destroy; override;

      // graphics can either be loaded on demand or use an already
      // loaded bitmap
      function   SetGraphic (Filename : String) : Boolean; overload;
      function   SetGraphic (Graphic : TBitMap) : Boolean; overload;
      
      procedure  SetCanvas  (Canvas : TCanvas);
      procedure  Initialise (Canvas : TCanvas; xW, yW, zW, wW, hW : Integer);
      procedure  Draw; virtual;
      function   GetScreenCoords : tPoint; virtual; abstract;

      property xWorld : Integer Read fxWorld Write fxWorld;
      property yWorld : Integer Read fyWorld Write fyWorld;
      property zWorld : Integer Read fxWorld;
      property wdthWorld : Integer Read fwdthWorld;
      property hghtWorld : Integer Read fhghtWorld;
      property Canvas : TCanvas Read fCanvas;
      property ClearGraphics  : Boolean Read fClearGraphics Write fClearGraphics;
      
  End;
      
implementation

constructor tGraphicObject.Create;
Begin
  fGraphic := tBitmap.Create;
  fClearGRaphics := FALSE;
End;

destructor tGRaphicObject.Destroy;
Begin
  if not fClearGRaphics Then exit;
  
  if Assigned (fGraphic) Then
    fGraphic.Free;
End;

function   tGraphicObject.SetGraphic (Graphic : TBitMap) : Boolean;
Begin
  Result := TRUE;
  fGraphic := Graphic;
  fClearGRaphics := FALSE;
End;

function   tGraphicObject.SetGraphic (Filename : String) : Boolean;
Begin
  try
    fGraphic.LoadFromFile(Filename);
    fClearGraphics := TRUE;
    Result := TRUE;
  except
    Result := FALSE;
  End;
End;

procedure  tGraphicObject.Initialise (Canvas : TCanvas; xW, yW, zW, wW, hW : Integer);
Begin
  SetCanvas (Canvas);
  fxWorld := xW;
  fyWorld := yW;
  fzWorld := zW;
  fwdthWorld := wW;
  fhghtWorld := hw;
End;
  
procedure  tGraphicObject.Draw;
var
  tmpCoord : TPoint;
  
Begin
  tmpCoord := GetScreenCoords;
  fCanvas.Draw(tmpCoord.X, tmpCoord.Y, fGraphic);
End;

procedure TGRaphicObject.SetCanvas (Canvas : TCanvas);
Begin
  fCanvas := Canvas;
End;
  
end.
