unit uMaze;

interface

uses SysUtils, QForms;

const 
  MAZE_SIZE = 97;
  MAZE_SIZE_IN_LOCS = 16; 

type

  // array to hold maze data
  tMazeData = array [0..MAZE_SIZE-1 , 0..MAZE_SIZE-1] of byte;
  
  // class for handling maze data
  TMolMaze = class 
    private
      fMaze : tMazeData;
      fMolManXpos : Integer;
      fMolManYPos : Integer;
      fMolGraphicNo   : Byte;
      fCircuitGraphicNo : Byte;

      
    public

      procedure ClearMaze;
      procedure SetMazeObject (xpos, ypos : Integer; MazeObject : Byte);
      procedure ClearMazeObject (xpos, ypos : Integer);
      function LoadMaze (Filename : String) : Boolean;
      function SaveMaze (Filename : String) : Boolean;
      function GetMazeObject (xpos, ypos : Integer) : Byte;
      
      procedure InGameDestroyObject (xpos, ypos : Integer);
      procedure UnDestroyAll;
      procedure GetMolManLocation;
      
      property MolManXPos : Integer Read fMolManXPos;
      property MolManYPos : Integer Read fMolManYPos;
      property MolGRaphicNo   : Byte Read FMolGraphicNo;
      property CircuitGraphicNo : Byte Read fCircuitGraphicNo Write fCircuitGraphicNo;

      function TotalNumCircuits : Integer;  
   End;
   
implementation

uses
  IniFiles;
  
procedure tMolMAze.ClearMaze;
var
  iLoop : Integer;
  iLoop1 : Integer;
  
begin
  for iLoop := 0 to MAZE_SIZE-1 do
    for iLoop1 := 0 to MAZE_SIZE-1 do
      fMaze[iLoop, iLoop1] := 0;
end;

procedure tMolMaze.SetMazeObject (xpos, ypos : Integer; MazeObject : Byte);
Begin
  fMaze [xPos, yPos] := MazeObject;
end;

procedure tMolMaze.ClearMazeObject(xpos, ypos : Integer);
begin
  fMaze [xPos, yPos] := 0;
end;

{$HINTS OFF}
function tMolMaze.LoadMaze (Filename : String) : Boolean;
var
  MazeFile : File of tMazeData;
  tmpMaze  : tMazeData;
  IniGraph  : TIniFile;
  
Begin
  Result := FALSE;
     
  IniGraph := TIniFile.Create (ExtractFilePath (Application.ExeName) + '/molgraphics.ini');
  fMolGraphicNo := IniGraph.ReadInteger('General','molmangraphicnum', 1);
  fCircuitGraphicNo := IniGraph.ReadInteger('General', 'circuitgraphicnum', 2);
  
  IniGraph.Free;
  
  try
    AssignFile (MazeFile, Filename);
    Reset (MazeFile);
    Read (MazeFile, tmpMaze);
    Result := TRUE;
    fMaze := tmpMaze;

    // we need to know where molecule man is
    GetMolManLocation;
  finally
    CloseFile (MazeFile);
  end;  
End;
{$HINTS ON}

{$HINTS OFF}
function tMolMaze.SaveMaze (Filename : String) : Boolean;
var
  MazeFile : File of tMazeData;
  
Begin
  Result := FALSE;

  try
    AssignFile (MazeFile, Filename);
    ReWrite (MazeFile);
    Write (MazeFile, fMaze);
    Result := TRUE;

  finally
    CloseFile (MazeFile);
  end;  
End;
{$HINTS ON}
         
function tMolMaze.GetMazeObject (xpos, ypos : Integer) : Byte;
begin
  Result := fMaze[xpos, ypos];
end;

procedure tMolMaze.GetMolManLocation;
var 
  iLoop1 : Integer;
  iLoop  : Integer;
    
begin
  
    // search through the maze, looking for Molecule Man
    For iLoop := 0 to MAZE_SIZE -1 do
      For iLoop1 := 0 to MAZE_SIZE -1 do
        if fMaze[iLoop, iLoop1] = FMolGraphicNo Then
        Begin
          FMolManXpos := iLoop;
          FMolManYPos := iLoop1;
          Exit;
        End;
End;

// used in the game to signify which objects have been destroyed and
// which coins have been picked up
procedure tMolMaze.InGameDestroyObject (xpos, ypos : Integer);
begin
   // set bit 7
  fMaze [xpos, ypos] := fMaze [xpos, ypos] OR 128;
End;

// undestroys all destroyed/picked up objects ready for a new game
procedure tMolMaze.UnDestroyAll;
var
  iLoop, iLoop1 : Integer;

Begin
  For iLoop := 0 to MAZE_SIZE-1 do
    for iLoop1 := 0 to MAZE_SIZE-1 do
      fMaze [iLoop1, iLoop] := fMaze [iLoop1,iLoop] AND 127; // reset bit 7
end;

function tMolMaze.TotalNumCircuits : Integer;  
var 
  iLoop1 : Integer;
  iLoop  : Integer;
    
begin
  
    Result := 0;
    
    // search through the maze, counting all the circuits
    For iLoop := 0 to MAZE_SIZE -1 do
      For iLoop1 := 0 to MAZE_SIZE -1 do
        if (fMaze[iLoop, iLoop1]) and 127 = fCircuitGraphicNo Then
          inc (Result);
End;
          
end.
 