unit uMolGameSetup;

interface

uses 
  IniFiles, SysUtils;
  
type
  // class to manage game configuration values
  tGameSetup = class
    private
      fTimeStartVal   : Integer;     // time remaining at game start
      fTimeCountDelay : Integer;     // how fast time counts down
      fStartPills     : Integer;     // no of pills at game start
      fPillCountDelay : Integer;     // how fast pills count down
      fPillsBought    : Integer;     // how many pills are bought at a time
      fStartBombs     : Integer;     // no of bombs at game start
      fStartCoins     : Integer;     // no of coins at game start
      fBombsBought    : Integer;     // how many bombs are bought at a time
      fUpLeftKey      : Char;        // key to move up and left
      
      fUpRightKey     : Char; 
      fDownRightKey   : Char; 
      fDownLeftKey    : Char; 
      fPauseKey       : Char;
      fAbortKey       : Char;
      fBuyKey         : Char;
      fGetKey         : Char;
      fBombKey        : Char;

    public
      property TimeStartVal    : Integer Read fTimeStartVal Write fTimeStartVal;
      property TimeCountDelay  : Integer Read fTimeCountDelay Write fTimeCountDelay;
      property StartPills      : Integer Read fStartPills Write fStartPills;
      property PillCountDelay  : Integer Read fPillCountDelay Write fPillCountDelay;
      property PillsBought     : Integer Read fPillsBought write fPillsBought;
      property StartBombs      : Integer Read fStartBombs write fStartBombs;
      property StartCoins      : Integer Read fStartCoins write fStartCoins;
      property BombsBought     : Integer Read fBombsBought write fBombsBought;
      property UpLeftKey       : Char Read fUpLeftKey Write fUpLeftKey;
      property UpRightKey      : Char Read fUpRightKey Write fUpRightKey;
      property DownRightKey    : Char Read fDownRightKey Write fDownRightKey;
      property DownLeftKey     : Char Read fDownLeftKey Write fDownLeftKey;
      property PauseKey        : Char Read fPauseKey Write fPauseKey;
      property AbortKey        : Char Read fAbortKey Write fAbortKey;
      property BuyKey          : Char Read fBuyKey Write fBuyKey;
      property GetKey          : Char Read fGetKey Write fGetKey;
      property BombKey         : Char Read fBombKey Write fBombKey;
      
      function ReadSetup (IniFileName : String) : Boolean;
      function SaveKeys  (IniFileName : String) : Boolean;
   End;

implementation

function tGameSetup.ReadSetup (iniFileName : String) : Boolean;
var
  IniSetup : TIniFile;

  function ReadChar (Section, key, default : String) : Char;
  var
    sTemp : String;

  Begin
    sTemp := IniSetup.ReadString (Section, key, Default);
    if Uppercase (sTemp)='SPACE' Then
      sTemp := ' ';
      
    Result := sTemp[1];
  End; 
  
begin
  
  try
    IniSetUp := TIniFile.Create(iniFileName);
  except
    Result := FALSE;
    exit;
  end;

  fTimeStartVal   := IniSetUp.ReadInteger ('Setup', 'starttime', 3300);
  fTimeCountDelay := IniSetup.ReadInteger ('Setup', 'timecountdown', 8);
  fStartPills     := IniSetup.ReadInteger ('Setup', 'startpills',20);
  fPillCountDelay := IniSetup.ReadInteger ('Setup', 'pillcountdown', 30);
  fPillsBought    := IniSetup.ReadInteger ('Setup', 'pillsbought', 20);
  fStartBombs     := IniSetup.ReadInteger ('Setup', 'startbombs', 0);
  fStartCoins     := IniSetup.ReadInteger ('Setup', 'startcoins', 0);
  fBombsBought    := IniSetup.ReadInteger ('Setup', 'bombsbought', 1);
  fUpLeftKey      := ReadChar  ('Keys', 'upleft', 'q');
  fUpRightKey     := ReadChar  ('Keys', 'upright', 'p');
  fDownLeftKey    := ReadChar  ('Keys', 'downleft', 'i');
  fDownRightKey   := ReadChar  ('Keys', 'downright', 'z');
  fBuyKey         := ReadChar  ('Keys', 'buy', 'b');
  fBombKey        := ReadChar  ('Keys', 'bomb', 'd');
  fGetKey         := ReadChar  ('Keys', 'get', 'g');
  fAbortKey       := ReadChar  ('Keys', 'abort', 'a');
  fPauseKey       := ReadChar  ('Keys', 'pause', ' ');
  
  IniSetup.Free;
  Result := TRUE;
end;

function tGameSetup.SaveKeys (iniFileName : String) : Boolean;
var
  IniSetup : TIniFile;

  procedure WriteString (Section, key, value : String);
  begin
    if value=' ' Then
      value := 'SPACE';
      
    IniSetup.WriteString(Section, key, value);
  End;
   
begin
  // save the keys setup back to the inifile
  try
    IniSetUp := TIniFile.Create(iniFileName);
  except
    Result := FALSE;
    exit;
  end;

  WriteString ('Keys', 'upleft', fUpLeftKey );
  WriteString ('Keys', 'upright', fUpRightKey);
  WriteString ('Keys', 'downleft', fDownLeftKey);
  WriteString ('Keys', 'downright', fDownRightKey);
  WriteString ('Keys', 'buy', fBuyKey);
  WriteString ('Keys', 'bomb', fBombKey);
  WriteString ('Keys', 'get', fGetKey);
  WriteString ('Keys', 'abort', fAbortKey);
  WriteString ('Keys', 'pause', fPauseKey);

  IniSetup.UpdateFile;
  IniSetup.Free;
  Result := TRUE;
  
end;

end.
 