unit uMolPanelSetup;

interface

uses
  IniFiles, Types, QGraphics, SysUtils, QForms, QDialogs;

type
  tPanelLine = record
    LineFrom    : TPoint;
    LineTo      : TPoint;
    Color       : TColor;
  End; 

  tPanelGraphic = record
    Graphic : TBitmap;
    XPos    : Integer;
    YPos    : Integer;
  End;

  tPanelText = Record
    Xpos               : Integer;
    yPos               : Integer;
    nDigts             : Integer;
    centerHorizontally : Boolean;
  End;
     
  tPanelSetup = class (TObject)
    private
      fNumLines        : Integer;
      fLineWidth       : Integer;
      fLines           : Array of tPanelLine;
      fTimeGraphic     : tPanelGraphic;
      fPillsGraphic    : tPanelGraphic;
      fBombsGraphic    : tPanelGraphic;
      fCircuitsGraphic : tPanelGraphic;
      fCoinsGraphic    : tPanelGraphic;
      fTextColor       : tColor;
      fTextHeight      : Integer;
      fTimeText        : tPanelText;
      fBombText        : tPanelText;
      fCoinsText       : tPanelText;
      fPillsText       : tPanelText;
      fCircuitsText    : tPanelText;

      procedure DrawNumericValue (iValue : Integer; PanelText : TPanelText; dstCanvas : TCanvas);
       
    public                    

      function ReadPanelSetup (IniFileName : String) : Boolean;
      procedure DrawLines (dstCanvas : TCanvas);
      procedure DrawGraphics (dstCanvas : TCanvas);
      procedure DrawTimeText (iValue : Integer; dstCanvas : tCanvas);
      procedure DrawBombText (iValue : Integer; dstCanvas : tCanvas);
      procedure DrawCoinsText (iValue : Integer; dstCanvas : tCanvas);
      procedure DrawCircuitsText (iValue : Integer; dstCanvas : tCanvas);
      procedure DrawPillsText (iValue : Integer; dstCanvas : tCanvas);
              
      destructor destroy; override;
  end;
  
implementation

function tPanelSetup.ReadPanelSetup(IniFileName : String) : Boolean;
var
  IniSetup : TIniFile;
  iLoop    : Integer;
  ptTemp   : TPoint;
  
Begin

  try
    IniSetup := TIniFile.Create(IniFilename);
  except
    Result := FALSE;
    Exit;
  End;

  fNumLines := IniSetup.ReadInteger ('PanelLines', 'numlines', 0);
  fLineWidth := IniSetup.ReadInteger('PanelLines','linewidth',4);
  fTextColor := IniSetup.ReadInteger('Text', 'color', 16777215);
  fTextHeight := IniSetUp.ReadInteger('Text', 'height', 14);
  
  SetLength (fLines, fNumLines);

  // read in the details of each line in turn
  for iLoop := 1 to fNumLines do
  Begin            
    ptTemp.X := IniSetup.ReadInteger('PanelLines','line'+IntToStr(iLoop)+'fromx',0);
    ptTemp.Y := IniSetup.ReadInteger('PanelLines','line'+IntToStr(iLoop)+'fromy',0);
    fLines[iLoop-1].LineFrom := ptTemp;
    ptTemp.X := IniSetup.ReadInteger('PanelLines','line'+IntToStr(iLoop)+'tox',0);
    ptTemp.Y := IniSetup.ReadInteger('PanelLines','line'+IntToStr(iLoop)+'toy',0);
    fLines[iLoop-1].LineTo := ptTemp;
    fLines[iLoop-1].Color := IniSetup.ReadInteger('PanelLines',
                                                  'line'+IntToStr(iLoop)+'colour',0);
  End;                                              

  fTimeGRaphic.Graphic := TBitMap.Create;
  fPillsGRaphic.Graphic := TBitMap.Create;
  fCoinsGRaphic.Graphic := TBitMap.Create;
  fBombsGRaphic.Graphic := TBitMap.Create;
  fCircuitsGRaphic.Graphic := TBitMap.Create;
  try
    fTimeGRaphic.Graphic.LoadFromFile(IniSetup.ReadString('TimeGraphic','filename','graphics/watch.xpm')); 
    fTimeGraphic.XPos := IniSetup.ReadInteger('TimeGraphic','xpos',224);
    fTimeGraphic.YPos := IniSetup.ReadInteger('TimeGraphic','ypos',300);
    fPillsGRaphic.Graphic.LoadFromFile(IniSetup.ReadString('PillsGraphic','filename','graphics/pillbottle.xpm')); 
    fPillsGraphic.XPos := IniSetup.ReadInteger('PillsGraphic','xpos',16);
    fPillsGraphic.YPos := IniSetup.ReadInteger('PillsGraphic','ypos',300);
    fCircuitsGRaphic.Graphic.LoadFromFile(IniSetup.ReadString('CircuitsGraphic','filename','graphics/panelcircuit.xpm')); 
    fCircuitsGraphic.XPos := IniSetup.ReadInteger('CircuitsGraphic','xpos',428);
    fCircuitsGraphic.YPos := IniSetup.ReadInteger('CircuitsGraphic','ypos',230);
    fBombsGRaphic.Graphic.LoadFromFile(IniSetup.ReadString('BombsGraphic','filename','graphics/panelbomb.xpm')); 
    fBombsGraphic.XPos := IniSetup.ReadInteger('BombsGraphic','xpos',16);
    fBombsGraphic.YPos := IniSetup.ReadInteger('BombsGraphic','ypos',230);
    fCoinsGRaphic.Graphic.LoadFromFile(IniSetup.ReadString('CoinsGraphic','filename','graphics/pound.xpm')); 
    fCoinsGraphic.XPos := IniSetup.ReadInteger('CoinsGraphic','xpos',428);
    fCoinsGraphic.YPos := IniSetup.ReadInteger('CoinsGraphic','ypos',300);
    
    fTimeText.Xpos := IniSetup.ReadInteger('Text', 'timex', 256);
    fTimeText.Ypos := IniSetup.ReadInteger('Text', 'timey', 365);
    fTimeText.nDigts := IniSetup.ReadInteger('Text', 'timendigits', 4);
    fTimeText.centerHorizontally := IniSetup.ReadBool ('Text', 'timecenterhorizontally', TRUE);
    fBombText.Xpos := IniSetup.ReadInteger('Text', 'bombx', 84);
    fBombText.Ypos := IniSetup.ReadInteger('Text', 'bomby', 270);
    fBombText.nDigts := IniSetup.ReadInteger('Text', 'bombndigits', 2);
    fBombText.centerHorizontally := IniSetup.ReadBool ('Text', 'bombcenterhorizontally', FALSE);
    fCoinsText.Xpos := IniSetup.ReadInteger('Text', 'coinsx', 404);
    fCoinsText.Ypos := IniSetup.ReadInteger('Text', 'coinsy', 340);
    fCoinsText.nDigts := IniSetup.ReadInteger('Text', 'coinsndigits', 2);
    fCoinsText.centerHorizontally := IniSetup.ReadBool ('Text', 'coinscenterhorizontally', FALSE);
    fCircuitsText.Xpos := IniSetup.ReadInteger('Text', 'circuitsx', 404);
    fCircuitsText.Ypos := IniSetup.ReadInteger('Text', 'circuitsy', 270);
    fCircuitsText.nDigts := IniSetup.ReadInteger('Text', 'circuitsndigits', 2);
    fCircuitsText.centerHorizontally := IniSetup.ReadBool ('Text', 'circuitscenterhorizontally', FALSE);
    fPillsText.Xpos := IniSetup.ReadInteger('Text', 'pillsx', 84);
    fPillsText.Ypos := IniSetup.ReadInteger('Text', 'pillsy', 340);
    fPillsText.nDigts := IniSetup.ReadInteger('Text', 'pillsndigits', 2);
    fPillsText.centerHorizontally := IniSetup.ReadBool ('Text', 'pillscenterhorizontally', FALSE);
      
  except
    Result := FALSE;
    exit;
  End;   
     
  Result := TRUE;
  IniSetup.Free;
end;

procedure tPanelSetup.DrawLines(dstCanvas : TCanvas);
var
  iLoop      : Integer;
  iOldWidth  : Integer;
  clOldColor : TColor;
 
Begin
  With dstCanvas do
  Begin
    iOldWidth := Pen.Width;
    clOldColor := Pen.Color;
    Pen.Width := fLineWidth;
  
    For iLoop :=1 to fNumLines do
    Begin
      Pen.Color := fLines[iLoop-1].Color;
      MoveTo(fLines[iLoop-1].LineFrom.x, fLines[iLoop-1].LineFrom.Y); 
      LineTo(fLines[iLoop-1].LineTo.x, fLines[iLoop-1].LineTo.Y); 
    End;

    Pen.Color := clOldColor;
    Pen.Width := iOldWidth;
  end;  
end;

destructor tPanelSetup.destroy;
Begin
  fTimeGraphic.Graphic.Free;
  fCoinsGraphic.Graphic.Free;
  fCircuitsGraphic.Graphic.Free;
  fBombsGraphic.Graphic.Free;
  fPillsGraphic.Graphic.Free;
  inherited;
End;

procedure tPanelSetup.DrawGraphics(dstCanvas : TCanvas);
Begin
  With fTimeGraphic do
    dstCanvas.Draw(XPos, YPos, Graphic); 

  With fPillsGraphic do
    dstCanvas.Draw(XPos, YPos, Graphic); 
    
  With fCircuitsGraphic do
    dstCanvas.Draw(XPos, YPos, Graphic); 

  With fBombsGraphic do
    dstCanvas.Draw(XPos, YPos, Graphic); 
    
  With fCoinsGraphic do
    dstCanvas.Draw(XPos, YPos, Graphic); 
    
End;

procedure tPanelSetup.DrawNumericValue (iValue : Integer;
                                        PanelText : TPanelText; dstCanvas : TCanvas);
var
  sNumericValue : String;
  iTextHeight   : Integer;
  iTextWidth    : Integer;
  clOldColor    : TColor;
  iOldSize    : Integer;
  
Begin

  With dstCanvas.Font do
  Begin
    clOldColor := Color;
    iOldSize := Size;
    Color := fTextColor;
    Size  := fTextHeight;
  End;
  
  sNumericValue := IntToStr (iValue);
  
  // expand to required length
  while Length (sNumericValue) < PanelText.nDigts Do
    sNumericValue := '0' + sNumericValue;

  //draw the text centred horizonally or vertically as required
  if PanelText.centerHorizontally Then
  Begin
    iTextWidth := dstCanvas.TextWidth(sNumericValue);
    dstCanvas.TextOut(PanelText.xpos-(iTextWidth div 2), 
                      PanelText.ypos, sNumericValue);
  End
  else
  Begin
    iTextHeight := dstCanvas.TextHeight(sNumericValue); 
    dstCanvas.TextOut (PanelText.xPos, 
                       PanelText.yPos- (iTextHeight div 2), sNumericValue);  
  End;

  With dstCanvas.Font do
  Begin
    Color := clOldColor;
    Size := iOldSize;
  End; 
End;

procedure tPanelSetup.DrawTimeText(iValue : Integer; dstCanvas : TCanvas);
Begin
  DrawNumericValue (iValue, fTimeText, dstCanvas);
end;

procedure tPanelSetup.DrawBombText(iValue : Integer; dstCanvas : TCanvas);
Begin
  DrawNumericValue (iValue, fBombText, dstCanvas);
end;

procedure tPanelSetup.DrawCoinsText(iValue : Integer; dstCanvas : TCanvas);
Begin
  DrawNumericValue (iValue, fCoinsText, dstCanvas);
end;

procedure tPanelSetup.DrawCircuitsText(iValue : Integer; dstCanvas : TCanvas);
Begin
  DrawNumericValue (iValue, fCircuitsText, dstCanvas);
end;

procedure tPanelSetup.DrawPillsText(iValue : Integer; dstCanvas : TCanvas);
Begin
  DrawNumericValue (iValue, fPillsText, dstCanvas);
end;

end.

