unit umazeviewer;

interface

uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, uMaze, QExtCtrls, QComCtrls, QMenus, QActnList,
  QImgList;

type
  TfrmMazeViewer = class(TForm)
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ScrollBox1: TScrollBox;
    Image1: TImage;
    ImageList1: TImageList;
    ActionList1: TActionList;
    actSave: TAction;
    actExit: TAction;
    actColor: TAction;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Save1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    View1: TMenuItem;
    BackgroundColour1: TMenuItem;
    PopupMenu1: TPopupMenu;
    Save2: TMenuItem;
    ColorDialog1: TColorDialog;
    SaveDialog1: TSaveDialog;
    actZoomin: TAction;
    actZoomOut: TAction;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    N2: TMenuItem;
    Zoomin1: TMenuItem;
    Zoomout1: TMenuItem;
    N3: TMenuItem;
    Zoomin2: TMenuItem;
    Zoomout2: TMenuItem;
    ImageList2: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure actColorExecute(Sender: TObject);
    procedure actExitExecute(Sender: TObject);
    procedure actSaveExecute(Sender: TObject);
    procedure actZoominExecute(Sender: TObject);
    procedure actZoomOutExecute(Sender: TObject);
  private
    { Private declarations }
    fMolMaze : TMolMaze;
    fGraphicList : TStrings;
    fBackColor   : TColor;
    fGraphicSize : Integer;
    
  public
    { Public declarations }
    property MolMaze : tMolMaze Read fMolMaze Write fMolMaze;
    property GraphicList : TStrings Write fGraphicList;
    
    procedure DrawMaze;
    
  end;

var
  frmMazeViewer: TfrmMazeViewer;

implementation

const
  MIN_GRAPHIC_SIZE=8;
  MAX_GRAPHIC_SIZE=32;

{$R *.xfm}

procedure TfrmMazeViewer.FormCreate(Sender: TObject);
begin
  fMolMaze := NIL;
  fGraphicList := NIL;
  fBackColor := clBlack;
  fGraphicSize := 16;
end;

procedure TfrmMazeViewer.DrawMaze;
var
  iLoop : Integer;
  iLoop1 : Integer;
  Rect   : TRect;
  PaintBox2 : TBitMap;
  
Begin
  if (not Assigned (fMolMaze)) or (not Assigned (fGraphicList)) then
    exit;

  Screen.Cursor := crHourglass;
    
  // the maze is drawn to an offscreen bitmap first
  PaintBox2 := TBitmap.Create;
  PaintBox2.Width := (fGraphicSize * MAZE_SIZE)-1;
  PaintBox2.Height := PaintBox2.Width;
  PAintBox2.Canvas.Brush.Color := fBackColor;
    
  PaintBox2.Canvas.CopyMode := cmSrcCopy;
  For iLoop := 0 to MAZE_SIZE -1 do
    For iLoop1 := 0 to MAZE_SIZE -1 do
    Begin
      // work out where to print the current object and what size it's going to 
      // be
      Rect.Left := iLoop1 * fGraphicSize;
      Rect.Right := Rect.Left + fGraphicSize -1;
      Rect.Top := iLoop * fGraphicSize;
      Rect.Bottom := Rect.Top + fGraphicSize -1;
      PaintBox2.Canvas.FillRect (Rect);

      if fMolMaze.GetMazeObject (iLoop1, iLoop) <> 0 Then
        PaintBox2.Canvas.StretchDraw (Rect, 
            fGraphicList.Objects [fMolMaze.GetMazeObject (iLoop1, iLoop)] As TGraphic);
    End;

   // save the bitmap to a temp file, and then load into Image1 for viewing
   PaintBox2.SaveToFile ('mazeview.xpm'); 
   Image1.Picture.LoadFromFile('mazeview.xpm');
   DeleteFile ('mazeview.xpm');
   PaintBox2.Free;

   Screen.Cursor := crDefault;
   
end;

procedure TfrmMazeViewer.actColorExecute(Sender: TObject);
begin
  if ColorDialog1.Execute Then
  Begin
    fBackColor := ColorDialog1.Color;
    DrawMaze;
  End;
  BringToFront;
end;

procedure TfrmMazeViewer.actExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfrmMazeViewer.actSaveExecute(Sender: TObject);
begin
  if SaveDialog1.Execute Then
    Image1.Picture.SaveToFile(SaveDialog1.FileName);
    
  BringToFront;
end;

procedure TfrmMazeViewer.actZoominExecute(Sender: TObject);
begin
  // increase the size of the graphics in the maze ...
  fGraphicSize := fGraphicSize * 2;
  actZoomin.Enabled := fGraphicSize <> MAX_GRAPHIC_SIZE;
  actZoomout.Enabled := TRUE;
  DrawMaze;
  
end;

procedure TfrmMazeViewer.actZoomOutExecute(Sender: TObject);
begin
  fGraphicSize := fGraphicSize div 2;
  actZoomout.Enabled := fGraphicSize <> MIN_GRAPHIC_SIZE;
  actZoomin.Enabled := TRUE;
  DrawMaze;
end;

end.
