unit umolman;

interface

uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, QExtCtrls, umolgraphics, iniFiles, uMaze, QComCtrls,
  QActnList, QImgList, QMenus, uMolGameSetup, uMolPanelSetup, umolsoundhandler;

const 
  VERSION = 'V0.91';
  
type
  TfrmMolMan = class(TForm)
    tmrGame: TTimer;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    Panel1: TPanel;
    PaintBox1: TPaintBox;
    ActionList1: TActionList;
    actNewGame: TAction;
    actPauseGame: TAction;
    actAbortGame: TAction;
    actExit: TAction;
    MainMenu1: TMainMenu;
    Game1: TMenuItem;
    NewGame1: TMenuItem;
    N1: TMenuItem;
    PauseUnPauseGame1: TMenuItem;
    AbortGame1: TMenuItem;
    Exit1: TMenuItem;
    imgactions: TImageList;
    actHelpAbout: TAction;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    Help1: TMenuItem;
    AboutMoleculeMan1: TMenuItem;
    actHelpInstructions: TAction;
    ToolButton10: TToolButton;
    N3: TMenuItem;
    AboutMoleculeMan2: TMenuItem;
    actLoadMaze: TAction;
    ToolButton11: TToolButton;
    ToolButton12: TToolButton;
    N2: TMenuItem;
    LoadMaze1: TMenuItem;
    N4: TMenuItem;
    OpenDialog1: TOpenDialog;
    actRedfineKeys: TAction;
    ToolButton13: TToolButton;
    ToolButton14: TToolButton;
    Optons1: TMenuItem;
    RedfineKeys1: TMenuItem;
    tmrAmbientSounds: TTimer;
    procedure tmrGameTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormDestroy(Sender: TObject);
    procedure actNewGameExecute(Sender: TObject);
    procedure actPauseGameExecute(Sender: TObject);
    procedure actAbortGameExecute(Sender: TObject);
    procedure actExitExecute(Sender: TObject);
    procedure actHelpAboutExecute(Sender: TObject);
    procedure actHelpInstructionsExecute(Sender: TObject);
    procedure actLoadMazeExecute(Sender: TObject);
    procedure actRedfineKeysExecute(Sender: TObject);
    procedure tmrAmbientSoundsTimer(Sender: TObject);

  private
    { Private declarations }
    fGraphicArray : tMolGraphicArray;
    fmolman  : TMolPlayerSprite;
    fGameGraphics : tGameGraphicsArray;
    fMolManGraphics : tMolManGraphicsArray;
    fGameSetup    : tGameSetup;
    fPanelSetup   : tPanelSetup;
    fMolSoundHandler : tMolSoundHandler;
    
    fKey     : Char;          // last key pressed
    fMolStartX : Integer;     // player start x pos in current location
    FMolStartY : Integer;     // player start x pos in current location
    FMolCurX   : Integer;     // player current x pos in location
    FMolCurY   : Integer;     // player current x pos in location
    FCurLocX   : Integer;     // x pos of current location
    FCurLocY   : Integer;     // y pos of current location
    fOldMazeObj : Byte;       // object at mol man location starting pos
    
    fOldFacing  : TMovementDir; 
    fTeleported : Boolean;
    fAborted    : Boolean;    
    fPaused     : Boolean;
    
    fTimeRemaining      : Integer;   // game time remaining
    fTimeCountdownDelay : Integer;   // time between game time decrements
    fCurTimeCountValue  : Integer;   // current time countdown value
    fNumPills           : Integer;   // numer of pills player has
    fPillsCountDownDelay: Integer;   // time between pill decrements
    fCurPillsCountValue : Integer;   // current pill countdown value
    fNumBombs           : Integer;
    fNumCircuits        : Integer;
    fNumCoins           : Integer;

    fMaze             : tMolMaze;    // maze data
    fBackgroundColors : Array of TColor;  // the colors used for location backgrounds
    fNumBackColors    : Integer;

    fPausedGraphic    : TBitMap;     

    procedure CheckGetKey (MolXPos, MolYPos : Integer);
    procedure CheckDestroyKey (MolXPos, MolYPos : Integer);
    procedure CheckAbortKey;
    procedure CheckMovementKeys (MolXPos, MolYPos : Integer);
    procedure CheckBuyKey (MolXPos, MolYPos : Integer);
    procedure DrawPanel (bmpDest : TBitMap);
    procedure SetUpLocation;
    procedure ShowEndGameMessage;
    procedure NewGame;
    procedure EndGame;
    
    { Public declarations }
  end;

var
   frmMolMan : TFrmMolMan;
   
implementation

{$R *.xfm}

uses
  uGRaphic, uAboutMolMan, uMolInstructions, uRedefineKeys, uCompat, usplashscreen;

var
  frmSplash : TFrmSplash;
    
procedure TfrmMolMan.tmrGameTimer(Sender: TObject);
var
  rctSrc  : TRect;
  rctDest : TRect;
  PaintBox2 : TBitmap;
  iMolX : Integer;
  iMolY : Integer;
  iBackColor : Integer;

begin

  // indicate new sounds may need to be played
  fMolSoundHandler.MagicNum := fMolSoundHandler.MagicNum+1;
  
  if (fnumPills=0) or (fTimeRemaining=0) or (not fMolMan.Moving And fTeleported) Then
  Begin
    if fTeleported Then
      fMolSoundHandler.PlayCompletedSound
    else
      fMolSoundHandler.PlayDeathSound;
    EndGame;
    exit;
  End;

  CheckAbortKey;
  if fAborted Then
  Begin
    fMolSoundHandler.PlayDeathSound;
    EndGame;
    Exit;
  End;

  // sort out x & ys
  iMolX := FMolMan.xWorld div MOL_MAX_GRAPHIC_SIZE;
  iMolY := FMolMan.yWorld div MOL_MAX_GRAPHIC_SIZE;

  // process keys
  if (fKey<> #0) and (not fPaused) Then
  Begin
    CheckMovementKeys (iMolx, iMoly);
    CheckDestroyKey (iMolx, iMoly);
    CheckBuyKey (iMolx, iMoly);
    CheckGetKey (iMolx, iMoly);
  End;

  if (Uppercase (fKey) = Uppercase (fGameSetup.PauseKey)) Then
    ActPauseGame.Execute;

  if not fPaused Then
  Begin
    if Assigned (FMolMan) Then
      if fMolMan.Moving Then
      begin
        fGraphicArray.Remove_Graphic(fMolMan);
        fMolMan.Move;
        fGraphicArray.Place_Graphic(fMolMan);
        fMolSoundHandler.PlayWalkSound;

      End;

    dec (fCurTimeCountValue);
    if fCurTimeCountValue = 0 Then
    Begin
      dec (fTimeRemaining);
      fCurTimeCountValue := fTimeCountdownDelay;
      fMolSoundHandler.PlayClockSound(fTimeRemaining);
    end;

    dec (fCurPillsCountValue);
    if fCurPillsCountValue = 0 Then
    Begin
      dec (fNumPills);
      fCurPillsCountValue := fPillsCountDownDelay;
    End;
  End;

  PaintBox2 := TBitmap.Create;
  PaintBox2.Width := PaintBox1.Width;
  PaintBox2.Height := PaintBox1.Height;

  // draw to off-screen paintbox, then make it visible
  // create an off screen paintbox to draw to
  rctSrc.Left := 0;
  rctSrc.Top := 0;
  rctSrc.Right := PaintBox1.Width;
  rctSrc.Bottom := PaintBox1.Height;
  rctDest.Top := 0;
  rctDest.Left :=0;
  rctDest.Bottom := rctSrc.Bottom;
  rctDest.Right := rctSrc.Right;

  // set background colour
  iBackColor := ((fCurLocX Div (MOL_LOCATION_SIZE-1)) +
                (fCurLocY Div (MOL_LOCATION_SIZE-1)))
                 AND (fNumBackColors-1);
  PaintBox2.Canvas.Brush.Color := fBackgroundcolors [iBackColor];
  PaintBox2.Canvas.FillRect(rctSrc);

  // draw the screen
  fGraphicArray.SetGraphicsCanvas(PaintBox2.Canvas);
  fGraphicArray.DrawGraphics (FMolMan.Facing);

  if not fPaused Then
    fGraphicArray.AnimateGraphics;

  // draw the panel area
  DrawPanel (PaintBox2);

  // draw the paused graphic
  if fPaused Then
    PaintBox2.Canvas.Draw((PAintBox1.Width div 2)- (fPausedGraphic.Width div 2),
                          (PaintBox1.Height div 2) - (fPausedGraphic.Height div 2),
                          fPausedGRaphic);

  // copy the off-screen bitmap onto the form
  PaintBox1.Canvas.CopyMode := cmSrcCopy;
  PaintBox1.Canvas.CopyRect (rctDest, PaintBox2.Canvas, rctSrc) ;

  PaintBox2.Free;

  // finally, check to see if the pause/unpause key has been pressed
  if (fkey<>#0) and (fkey='5') Then
    actPauseGameExecute (Self);

  fKey := #0; // clear key press

end;

procedure TfrmMolMan.FormCreate(Sender: TObject);
var
  iniFile : TIniFile;
  iLoop   : Integer;

begin

  Randomize;

  // get the graphics from the splash screen from
  fGameGraphics := frmSplash.GameGraphics;
  fMolManGraphics := frmSplash.MolManGraphics;

  // load the maze
  fMaze := TMolMaze.Create;
  fMaze.LoadMaze (RespecFileName (ExtractFilePath (Application.ExeName) + '/molman.maze'));
  
  // read in background colour details
  iniFile := TIniFile.Create (ExtractFilePAth (Application.ExeName)+'/molgraphics.ini');
  fNumBackColors := IniFile.ReadInteger('backgroundcolours','numcolours',1);
  SetLength (fBackgroundColors, fNumBackColors);
  For iLoop := 0 to fNumBackColors-1 do
    fBackgroundColors[iLoop] := StrToInt 
                              (IniFile.ReadString ('backgroundcolours',
                                                   'colour'+ IntToStr (iLoop+1), '0'));
  IniFile.Free;                                                 

  Panel1.Color := fBackgroundColors[0];
  
  // load the paused graphic
  fPausedGraphic := TBitMap.Create;
  fPausedGraphic.LoadFromFile(RespecFileName (ExtractFilePath (
                                             Application.ExeName)+'/graphics/paused.xpm'));

  fGameSetup := tGameSetup.Create;
  if not fGameSetUp.ReadSetup(RespecFilename (ExtractFilePath (
                                              Application.Exename)+'/molman.ini')) Then
  Begin
    MessageDlg ('Fatal Error', 'Could not read initialisation file molman.ini'#13#10+
                'Molecule Man will now close', mtError, [mbOk], 0);
    Halt;
  End;

  fPanelSetup := tPanelSetup.Create;
  if not fPanelSetUp.ReadPanelSetup(RespecFilename (ExtractFilePath (
                                                    Application.ExeName)+'/panel.ini')) Then
  Begin
    MessageDlg ('Fatal Error', 'Could not read initialisation file panel.ini'#13#10+
                'Molecule Man will now close', mtError, [mbOk], 0);
    Halt
  End;

  fMolSoundHandler := tMolSoundHandler.Create;
  fMolSoundHandler.ReadMolSounds
      (RespecFilename (ExtractFilePath (Application.Exename) + '/sounds.ini'));


end;

procedure TfrmMolMan.FormKeyPress(Sender: TObject; var Key: Char);
begin
  fKey := Key;
end;

procedure TfrmMolMan.SetUpLocation;
var
  tmpBlock : tMolStaticSprite;
  iLoop : Integer;
  iLoop1 : Integer;
  iniGraph : TIniFile;
  iGraphic : Integer;
  tmpDest  : Boolean;
  tmpAnimDelay : Integer;
  tmpAnims     : Integer;
  tmpWidth     : Integer;
  tmpHeight    : Integer;
  tmpPillMachine : Boolean;
  tmpBombMachine : Boolean;
  tmpCoin        : Boolean;
  tmpCircuit     : Boolean;
  sSection       : String;
  fAnim1 : TMolAnimatedSprite;
  iMaxGraphic : Integer;
  iBackColor  : Integer;
  tmpTeleporter : Boolean;

Begin

  tmrGame.Enabled := FALSE;

  // clear out any existing graphics
  if Assigned (fGraphicArray) Then
  Begin
    fGraphicArray.FreeGRaphics;
    fGraphicArray.Free;
  End;

  // work out the background colour for this location
  iBackColor := ((fCurLocX Div (MOL_LOCATION_SIZE-1)) +
                (fCurLocY Div (MOL_LOCATION_SIZE-1))) 
                 AND (fNumBackColors-1);
  Panel1.Color := fBackgroundcolors [iBackColor];

  // save the maze object at the position of the player - prevent destroyed
  // objects being overwritten
  fOldMazeObj := fMaze.GetMazeObject(fMolCurX, fMolCurY);

  // drop the player into the current location
  fMaze.SetMazeObject(fMolCurX, fMolCurY, 7);

  fKey := #0;

  iniGraph := TIniFile.Create (ExtractFilePAth (Application.Exename) +'molgraphics.ini');
  iMaxGraphic := IniGraph.ReadInteger ('General', 'numgraphics', 0);
  
  fGraphicArray := TMolGRaphicArray.Create;
  
  // read the maze data for this location, and create graphicobjects as we go ..
  // N.B. ignore graphicobjects of 0 (i.e nothing there) and destroyed object
  For iLoop := 0 to MOL_LOCATION_SIZE-1 do
    For iLoop1 := 0 To MOL_LOCATION_SIZE-1 do
      if (fMaze.GetMazeObject (fCurLocX+iLoop1, fCurLocY+iLoop)<> 0) and
         ((fMaze.GetMazeObject (FCurLocx+iLoop1, fCurLocY+iLoop) And 128)<>128) Then
      Begin

        // a graphic is here, so get it's value
        iGraphic := fMaze.GetMazeObject (fCurLocX+iLoop1, fCurLocY+iLoop);

        // if it's not valid, set a safe default
        if iGraphic > iMaxGraphic Then
          iGraphic := 5;
          
        // a graphic object is at this location, so read it's data
        sSection := 'MolGraphic'+IntToStr(iGraphic);
        if IniGraph.ReadBool (sSection, 'moleculeman', FALSE) then
        begin
          // molecule man - 
          tmpWidth := IniGraph.ReadInteger (sSection, 'width', 8);
          tmpHeight := IniGraph.ReadInteger (sSection, 'height', 8);

          // create the molecule man sprite and initialise it
          fMolMan := TMolPlayerSprite.Create;
          fMolMan.SetGraphics(fMolManGraphics);  // graphics are already in memory
          fMolMan.AnimFrame := 1;
          fMolMan.Initialise(PaintBOx1.Canvas, iLoop1*8, iLoop*8, 0, 
                             tmpWidth, tmpHeight);
          fMolMan.Facing := fOldFacing;
          fGraphicArray.Place_Graphic(fMolMan);
          
        end
        else
        Begin
          if IniGraph.ReadBool (sSection, 'animated', FALSE) Then
          Begin
            // an animated sprite - read its details ...
            tmpDest := IniGraph.ReadBool(sSection, 'destructible', TRUE);
            tmpWidth := IniGraph.ReadInteger (sSection, 'width', 8);
            tmpHeight := IniGraph.ReadInteger (sSection, 'height', 8);
            tmpAnims := IniGraph.ReadInteger (sSection, 'numframes', 1);
            tmpAnimDelay := IniGraph.ReadInteger (sSection, 'animdelay', 2);
            tmpCircuit := IniGraph.ReadBool (sSection, 'circuit', FALSE); 
          
            fAnim1 := TMolAnimatedSprite.Create;
            fAnim1.SetGraphics(fGameGRaphics [iGraphic]) ; // graphics are already
                                                           // in memory
            // rest of initialisation                                               
            fAnim1.Destructible := tmpDest;
            fAnim1.IsCircuit := tmpCircuit;
            fAnim1.SetAnimInfo( tmpAnims, tmpAnimDelay);
            fAnim1.Initialise(PaintBox1.Canvas, iLoop1*8, iLoop*8, 0, tmpWidth, tmpHeight);
            fGraphicArray.Place_Graphic(fAnim1);
          
          end
          else
          Begin
            // static sprite - read its details
            tmpDest := IniGraph.ReadBool(sSection, 'destructible', TRUE);
            tmpWidth := IniGraph.ReadInteger (sSection, 'width', 8);
            tmpHeight := IniGraph.ReadInteger (sSection, 'height', 8);
            tmpPillMachine := IniGraph.ReadBool (sSection, 'pillmachine', FALSE);
            tmpBombMachine := IniGraph.ReadBool (sSection, 'bombmachine', FALSE);
            tmpCoin        := IniGraph.ReadBool (sSection, 'coin', FALSE);
            tmpTeleporter := IniGraph.ReadBool (sSection, 'teleporter',FALSE);
  
            // set it up ...
            tmpBlock := tMolStaticSprite.Create;
            if tmpBlock.SetGraphic( fGameGraphics[iGraphic,0]) Then
            Begin
  
              tmpBlock.Initialise(PaintBox1.Canvas,  iLoop1*8,
                                                     iLoop *8, 0,
                                                     tmpWidth, tmpHeight);
              tmpBlock.isPillMachine := tmpPillMachine;
              tmpBLock.Destructible := tmpDest;
              tmpBlock.isBombMachine := tmpBombMachine;
              tmpBlock.IsCoin := tmpCoin;
              tmpBlock.IsTeleporter := tmpTeleporter;
                              
              fGraphicArray.Place_GRaphic (tmpBlock);
            
            End;
          
          End;   
        End;
      End;
  IniGraph.Free;

  tmrGame.Enabled := TRUE;
End;

procedure TfrmMolMan.FormDestroy(Sender: TObject);
begin
  // tidy up
  fMaze.Free;
  fGameSetup.Free;
  fPanelSetup.Free;
  fPausedGraphic.Free;
end;

procedure TFrmMolMan.ShowEndGameMessage;
Begin

  if fAborted Then
  Begin
    MessageDlg ('Game Over', 'You have failed in your quest.'#13#10#13#10+
                'You collected '+IntToStr (fNumCircuits)+ ' circuits.',
                 mtInformation, [mbOk], 0);
    Color := clBackground;            
    PaintBox1.Visible := FALSE;
    exit;
  End;  
  
  if fTeleported Then
  Begin
    MessageDlg ('Game Over', 'You have escaped from the maze!'#13#10#13#10+
                'You collected all '+IntToStr (fNumCircuits)+ ' circuits and had '+
                IntToStr (fTimeRemaining) +' time remaining.', mtInformation, [mbOk], 0);
    Color := clBackground;            
    PaintBox1.Visible := FALSE;
    exit;
  End;  
  
  if fNumPills=0 Then
  Begin
    MessageDlg ('Game Over', 'You have failed in your quest.'#13#10#13#10+
                'You collected '+IntToStr (fNumCircuits)+ ' circuits and died '+
                'because you ran out of pills.', mtInformation, [mbOk], 0);
    Color := clBackground;            
    PaintBox1.Visible := FALSE;
    exit;
  End;  

  if fTimeRemaining=0 Then
  Begin
    MessageDlg ('Game Over', 'You have failed in your quest.'#13#10#13#10+
                'You collected '+IntToStr (fNumCircuits)+ ' circuits and died '+
                'because you ran out of time.', mtInformation, [mbOk], 0);
    Color := clBackground;            
    PaintBox1.Visible := FALSE;
    exit;
  End;  

  
end;

// do all the stuff required for a new game ....
procedure tFrmMolMan.NewGame;
Begin

  // undestroy any destroyed maze objects from the last game
  fMaze.UnDestroyAll; 
  
  fTeleported := FALSE;

  // work out the starting position of the player in the maze
  FMolStartX := fMaze.MolManXPos;
  FMolStartY := fMaze.MolManYPos;
  FMolCurX   := FMolStartX;
  FMolCurY   := FMolStartY;
  fCurLocX := (FMolCurX div (MOL_LOCATION_SIZE-1)) * (MOL_LOCATION_SIZE-1) ;
  fCurLocY := (FMolCurY div (MOL_LOCATION_SIZE-1)) * (MOL_LOCATION_SIZE-1) ;
  fMaze.SetMazeObject(fMolStartX, fMolStartY, 0);
  fOldFacing :=tmdDownRight;

  PaintBox1.Visible := TRUE;

  // set up initial values for the various game counters
  fTimeRemaining := fGameSetup.TimeStartVal;
  fTimeCountdownDelay := fGameSetup.TimeCountDelay;
  fCurTimeCountValue  := fTimeCountDownDelay;
  fNumPills           := fGameSetup.StartPills;
  fPillsCountDownDelay:= fGameSetup.PillCountDelay;
  fCurPillsCountValue := fPillsCountDownDelay;
  fNumBombs           := fGameSetup.StartBombs;
  fNumCoins           := fGameSetup.StartCoins;
  fNumCircuits        := 0; //always start with no circuits

  SetUpLocation;
  fAborted := FALSE;
  fPaused := FALSE;

  // start the timer that causes graphics to be drawn, keys scanned etc ..
  tmrGame.Enabled := TRUE;

  // setup and start the timer that plays ambient sounds at random intervals
  With fMolSoundHandler Do
    tmrAmbientSounds.Interval := (Random (MaxAmDelay - MinAmDelay) + MinAmDelay)*1000;
  tmrAmbientSounds.Enabled := TRUE;

End;

procedure TfrmMolMan.actNewGameExecute(Sender: TObject);
begin
  ActNewGame.Enabled := FALSE;
  ActLoadMaze.Enabled := FALSE;
  actPauseGame.Enabled := TRUE;
  actAbortGame.Enabled := TRUE;
  NewGame;
end;

procedure TfrmMolMan.actPauseGameExecute(Sender: TObject);
begin
  fPAused := not fPaused;
  ToolButton3.Down := fPaused;
end;

procedure TfrmMolMan.actAbortGameExecute(Sender: TObject);
begin
  fAborted := TRUE;
  EndGame;
end;

procedure TfrmMolMan.actExitExecute(Sender: TObject);
begin
  if not (actNewGame.Enabled) Then
    if MessageDlg ('Warning', 'You''re in the middle of a game.'#13#10#13#10+
                   'Are you sure you want to quit?', mtWarning, [mbYes, mbNo],0)
                   = mrYes Then
     Close;
end;

procedure TFrmMolMan.EndGame;
Begin
  // stop the game
  tmrGame.Enabled := FALSE;
  tmrAmbientSounds.Enabled := FALSE;

  fMaze.SetMazeObject(fMolCurX, fMolCurY, fOldMazeObj);
  ShowEndGameMessage;
  actNewGame.Enabled := TRUE;
  actLoadMaze.Enabled := TRUE;
  actPauseGame.Enabled := FALSE;
  actAbortGame.Enabled := FALSE;

  Panel1.Color := fBackgroundColors[0];
  
end;

procedure TFrmMolMan.CheckAbortKey;
Begin
  if (fKey<> #0) and (not fPaused) Then
    fAborted := uppercase (fkey) = Uppercase (fGameSetup.AbortKey);
End;

procedure TFrmMolMan.CheckMovementKeys (MolXPos, MolYPos : Integer);
var
  tmpGRaphic : TGraphicObject;

Begin
  if UpperCase (fKey) = Uppercase (fGameSetup.UpLeftKey) Then
    case fMolMan.Facing of
      tmdUpLeft    : if not fMolMan.Moving Then
                     Begin
                       // we're already facing in the right direction, so
                       //get the graphic fMolMan is facing
                       tmpGraphic := fGraphicArray.GetGraphic(MolXPos, MolYPos-1);

                       // if we're not on the edge of the screen, start moving
                       // otherwise we need to change location
                       if MolYPos <> 0 Then
                       Begin
                           // if there's no object in the way, start moving
                           if (tmpGraphic = NIL) Then
                             fMolMan.StartMoving;

                           // check if we're stepping onto the teleporter  
                           if ((tmpGraphic is TMolStaticSprite) AND
                              (tmpGraphic As TMolStaticSprite).IsTeleporter AND
                               (fNumCircuits=fMaze.TotalNumCircuits)) Then
                           Begin    
                            fMolMan.StartMoving;
                            fTeleported := TRUE;
                           End;

                           if not fMolMan.Moving Then
                             fMolSoundHandler.PlayNoMoveSound;
                       End
                       else
                         if fCurLocY <> 0 Then
                         Begin
                           // restore maze object at the postion we entered the
                           // location
                           fMaze.SetMazeObject(fMolCurX, fMolCurY, fOldMazeObj);

                           // work out where the next location is, and our position
                           // within it
                           fCurLocY := fCurLocY - (MOL_LOCATION_SIZE-1);
                           fMolCurY := fCurLocY + MOL_LOCATION_SIZE-1;
                           fMolCurX := fCurLocX + MolXPos;

                           fOldFacing := FMolMan.Facing;

                           // draw the new location
                           SetupLocation;

                           // setting the location can take time, so clear
                           // any pending timer messages
                           Application.ProcessMessages;
                         End;

                     End;

      // we're not facing in the right direction, so rotate ....
      tmdUpRight,
      tmdDownLeft  : if not fMolMan.Moving Then fMolMan.Facing := tmdUpLeft;
      tmdDownRight : if not fMolMan.Moving Then fMolMan.Facing := tmdUpRight;
    end;

  if UpperCase (fKey) = Uppercase (fGameSetup.DownLeftKey) Then
    case fMolMan.Facing of
      tmdDownLeft    : if not fMolMan.Moving Then
                         if MolXPos = 0 Then
                         Begin
                           if fCurLocX <> 0 Then
                           Begin
                             fMaze.SetMazeObject(fMolCurX, fMolCurY, fOldMazeObj);
                             fOldFacing := FMolMan.Facing;
                             fCurLocX := fCurLocX - (MOL_LOCATION_SIZE-1);
                             fMolCurX := fCurLocX + (MOL_LOCATION_SIZE-1);
                             fMolCurY := fCurLocY +MolYPos;
                             SetupLocation;
                             Application.ProcessMessages;
                           End;
                         End
                         else
                         Begin
                           tmpGraphic := fGraphicArray.GetGraphic(MolXPos-1, MolYPos);
                           if (tmpGraphic = NIL) Then
                             fMolMan.StartMoving;

                           if ((tmpGraphic is TMolStaticSprite) AND
                              (tmpGraphic As TMolStaticSprite).IsTeleporter AND
                              (fNumCircuits=fMaze.TotalNumCircuits)) Then
                           Begin
                             fMolMan.StartMoving;
                             fTeleported := TRUE;
                           End;

                           if not fMolMan.Moving Then
                             fMolSoundHandler.PlayNoMoveSound;

                         End;
      tmdUpLeft,
      tmdDownRight   : if not fMolMan.Moving Then fMolMan.Facing := tmdDownLeft;
      tmdUpRight     : if Not fMolMan.Moving Then fMolMan.Facing := tmdUpLeft;
    end;

  if UpperCase (fKey) = Uppercase (fGameSetup.DownRightKey) Then
    case fMolMan.Facing of
      tmdDownRight    : if not fMolMan.Moving Then
                        Begin
                          if MolYPos <> MOL_LOCATION_SIZE-1 THen
                          Begin
                            tmpGraphic := fGraphicArray.GetGraphic(MolXPos, MolYPos+1);
                             if (tmpGraphic = NIL) Then
                               fMolMan.StartMoving;

                             if ((tmpGraphic is TMolStaticSprite) AND
                                (tmpGraphic As TMolStaticSprite).IsTeleporter AND
                                (fNumCircuits=fMaze.TotalNumCircuits)) Then
                             Begin
                               fMolMan.StartMoving;
                               fTeleported := TRUE;
                             End;

                             if not fMolMan.Moving Then
                               fMolSoundHandler.PlayNoMoveSound;

                          end
                          else
                            if fCurLocY + MOL_LOCATION_SIZE -1 <> MAZE_SIZE-1 Then
                            Begin

                              // restore old object here
                              fMaze.SetMazeObject(fMolCurX, fMolCurY, fOldMazeObj);
                              fOldFacing := FMolMan.Facing;
                              fCurLocY := fCurLocY + MOL_LOCATION_SIZE-1;
                              fMolCurY := fCurLocY;
                              fMolCurX := fCurLocX + MolXPos;
                              SetupLocation;
                              Application.ProcessMessages;
                            End;

                        End;
      tmdUpRight,
      tmdDownLeft  : if not fMolMan.Moving Then fMolMan.Facing := tmdDownRight;
      tmdUpLeft    : if not fMolMan.Moving Then fMolMan.Facing := tmdUpRight;
    end;

  if UpperCase (fKey) = Uppercase (fGameSetup.UpRightKey) Then
    case fMolMan.Facing of
      tmdUpRight   : if not fMolMan.Moving Then
                       if MolXPos = MOL_LOCATION_SIZE-1 Then
                       Begin
                         if fCurLocY + MOL_LOCATION_SIZE-1 <> MAZE_SIZE-1 Then
                         Begin

                           fMaze.SetMazeObject(fMolCurX, fMolCurY, fOldMazeObj);
                           fOldFacing := FMolMan.Facing;
                           fCurLocX := fCurLocX + (MOL_LOCATION_SIZE-1);
                           fMolCurX := fCurLocX;
                           fMolCurY := fCurLocY + MolYPos;

                           SetupLocation;
                           Application.ProcessMessages;
                         End;
                       End
                       else
                       Begin
                         tmpGraphic := fGraphicArray.GetGraphic(MolXPos+1, MolYPos);
                         if (tmpGraphic = NIL) then
                          fMolMan.StartMoving;

                         if ((tmpGraphic is TMolStaticSprite) AND
                            (tmpGraphic As TMolStaticSprite).IsTeleporter AND
                            (fNumCircuits=fMaze.TotalNumCircuits)) Then
                         Begin
                           fMolMan.StartMoving;
                           fTeleported := TRUE;
                         End;

                         if not fMolMan.Moving Then
                           fMolSoundHandler.PlayNoMoveSound;
                         
                       End;

      tmdUpLeft,
      tmdDownRight   : if not fMolMan.MOving Then fMolMan.Facing := tmdUpRight;
      tmdDownLeft    : if not fMolMan.MOving Then fMolMan.Facing := tmdUpLeft;
    end;

End;

procedure TFrmMolMan.CheckDestroyKey (MolXPos, MolYPos : Integer);
var
  tmpGRaphic : TGraphicObject;

Begin
  if Uppercase (fkey)= Uppercase (fGameSetup.BombKey) Then
  Begin
    // the destroy key has been pressed ...
    
    case fMolMan.Facing of
                     // can't destroy if we're on the edge of the screen or
                     // have no bombs
      tmdDownRight : if (MolYPos <> MOL_LOCATION_SIZE -1) and (fNumBombs<>0) Then
                      Begin
                        // see if we're facing a graphic
                        tmpGraphic := fGraphicArray.GetGraphic(MolXPos, MolYPos+1);

                        if Assigned (tmpGraphic) Then
                        Begin
                          // launch a bomb
                          dec (fNumBombs);
                          fMolSoundHandler.PlayBombSound;

                          // remove destructible objects from the location,
                          // and set the destroyed flag in the maze
                          if (tmpGraphic As TMolBaseStaticSprite).Destructible Then
                          Begin
                            fMaze.InGameDestroyObject(fCurLocX + MolXPos, fCurLocY+MolYPos+1);
                            fGraphicArray.Remove_Graphic(tmpGraphic);
                          End;
                        End;
                      End;

      tmdUpLeft : if (MolYPos <> 0) and (fNumBombs<>0) Then
                      Begin
                        tmpGraphic := fGraphicArray.GetGraphic(MolXPos, MolYPos-1);
                        if Assigned (tmpGraphic) Then
                        Begin
                          fMolSoundHandler.PlayBombSound;
                          dec (fNumBombs);
                          if (tmpGraphic As tMolBaseStaticSprite).Destructible Then
                          Begin
                            fMaze.InGameDestroyObject(fCurLocX +MolXPos, fCurLocY+MolYPos-1);
                            fGraphicArray.Remove_Graphic(tmpGraphic);
                          End;
                        End;
                      End;

      tmdUpRight : if (MolXPos <> MOL_LOCATION_SIZE-1) and (fNumBombs<>0) Then
                      Begin
                        tmpGraphic := fGraphicArray.GetGraphic(MolXPos+1, MolYPos);
                        if Assigned (tmpGraphic) Then
                        Begin
                          fMolSoundHandler.PlayBombSound;
                          dec (fNumBombs);
                          if (tmpGraphic As tMolBaseStaticSprite).Destructible Then
                          Begin
                            fMaze.InGameDestroyObject(fCurLocX +MolXPos+1, fCurLocY+MolYPos);
                            fGraphicArray.Remove_Graphic(tmpGraphic);
                          End;
                        End;
                      End;

      tmdDownLeft : if (MolXPos <> 0) and (fNumBombs<>0)  Then
                      Begin
                        tmpGraphic := fGraphicArray.GetGraphic(MolXPos-1, MolYPos);
                        if Assigned (tmpGRaphic) Then
                        Begin
                          fMolSoundHandler.PlayBombSound;
                          dec (fNumBombs);
                          if (tmpGraphic As tMolBaseStaticSprite).Destructible Then
                          Begin
                            fMaze.InGameDestroyObject(fCurLocX+ MolXPos-1, fCurLocY+MolYPos);
                            fGraphicArray.Remove_Graphic(tmpGraphic);
                          End;
                        End;
                      End;

    end;
  end;

End;

procedure TFrmMolMan.CheckBuyKey(MolXPos, MolYPos : Integer);
var
  tmpGRaphic : TGraphicObject;

Begin
  if Uppercase (fkey)= Uppercase (fGameSetup.BuyKey) Then
  Begin
    tmpGraphic := NIL;

    // get the graphic object we're currently facing
    case fMolMan.Facing of
      tmdDownRight : if MolYPos <> MOL_LOCATION_SIZE -1 Then
                        tmpGraphic := fGraphicArray.GetGraphic(MolXPos, MolYPos+1);

      tmdUpLeft    : if MolYPos <> 0 Then
                      tmpGraphic := fGraphicArray.GetGraphic(MolXPos, MolYPos-1);
                        
      tmdUpRight   : if MolXPos <> MOL_LOCATION_SIZE-1 Then
                        tmpGraphic := fGraphicArray.GetGraphic(MolXPos+1, MolYPos);

      tmdDownLeft  : if MolXPos <> 0 Then
                        tmpGraphic := fGraphicArray.GetGraphic(MolXPos-1, MolYPos);
                        
    end;

    // are we facing anything ?
    if (Assigned (tmpGraphic)) and (tmpGraphic is tMolStaticSprite) Then
    Begin
      // pill machine
      if (tmpGraphic As tMolStaticSprite).IsPillMachine Then
        if fNumCoins<>0 Then
        Begin
          dec (fNumCoins);
          fNumPills := fNumPills+ fGameSetup.PillsBought;
          fMolSoundHandler.PlayBuySound;
        End;
        
      // bomb machine  
      if (tmpGraphic As tMolStaticSprite).IsBombMachine Then
        if fNumCoins<>0 Then
        Begin
          dec (fNumCoins);
          fNumBombs := fNumBombs + fGameSetup.BombsBought;
          fMolSoundHandler.PlayBuySound;
        End;
    End;
  end;
End;

procedure TFrmMolMan.CheckGetKey(MolXPos, MolYPos : Integer);
var
  tmpGRaphic : TGraphicObject;

begin

 if Uppercase (fkey)=Uppercase (fGameSEtup.GetKey) Then
 Begin
   case fMolMan.Facing of
     tmdDownRight : if MolYPos <> MOL_LOCATION_SIZE -1 Then
                     Begin
                       // get the object we're facing
                       tmpGraphic := fGraphicArray.GetGraphic(MolXPos, MolYPos+1);
                       
                       // coin
                       if (tmpGraphic is tMolStaticSprite) Then
                         if (tmpGraphic As tMolStaticSprite).IsCoin Then
                         Begin
                           inc (fNumCoins);
                           // remove the circuit from the location and the maze
                           fMaze.InGameDestroyObject(fCurLocX +MolXPos, fCurLocY+MolYPos+1);
                           fGraphicArray.Remove_Graphic(tmpGraphic);
                           fMolSoundHandler.PlayGetCoinSound;
                         End;

                       // circuit
                       if (tmpGraphic is tMolAnimatedSprite) Then
                         if (tmpGraphic As tMolAnimatedSprite).IsCircuit Then
                         Begin
                           inc (fNumCIrcuits);

                           // remove the circuit from the location and the maze
                           fMaze.InGameDestroyObject(fCurLocX + MolXPos, fCurLocY+MolYPos+1);
                           fGraphicArray.Remove_Graphic(tmpGraphic);
                           fMolSoundHandler.PlayGetCircuitSound;
                         End;

                     End;

     tmdUpLeft : if MolXPos <> 0 Then
                     Begin
                       tmpGraphic := fGraphicArray.GetGraphic(MolXPos, MolYPos-1);

                       if (tmpGraphic is tMolStaticSprite) Then
                         if (tmpGraphic As tMolStaticSprite).IsCoin Then
                         Begin
                           inc (fNumCoins);
                           fMaze.InGameDestroyObject(fCurLocX + MolXPos, fCurLocY+ MolYPos-1);
                           fGraphicArray.Remove_Graphic(tmpGraphic);
                           fMolSoundHandler.PlayGetCoinSound;
                         End;

                       if (tmpGraphic is tMolAnimatedSprite) Then
                         if (tmpGraphic As tMolAnimatedSprite).IsCircuit Then
                         Begin
                           inc (fNumCircuits);
                           fMaze.InGameDestroyObject(fCurLocX + MolXPos , fCurLocY+MolYPos-1);
                           fGraphicArray.Remove_Graphic(tmpGraphic);
                           fMolSoundHandler.PlayGetCircuitSound;
                         End;

                     End;

     tmdUpRight : if MolXPos <> MOL_LOCATION_SIZE-1 Then
                     Begin
                       tmpGraphic := fGraphicArray.GetGraphic(MolXPos+1, MolYPos);

                       if (tmpGraphic is tMolStaticSprite) Then
                         if (tmpGraphic As tMolStaticSprite).IsCoin Then
                         Begin
                           inc (fNumCoins);
                           fMaze.InGameDestroyObject(fCurLocX + MolXPos+1, fCurLocY+MolYPos);
                           fGraphicArray.Remove_Graphic(tmpGraphic);
                           fMolSoundHandler.PlayGetCoinSound;
                         End;

                       if (tmpGraphic is tMolAnimatedSprite) Then
                         if (tmpGraphic As tMolAnimatedSprite).IsCircuit Then
                         Begin
                           inc (fNumCircuits);
                           fMaze.InGameDestroyObject(fCurLocX + MolXPos+1, fCurLocY+MolYPos);
                           fGraphicArray.Remove_Graphic(tmpGraphic);
                           fMolSoundHandler.PlayGetCircuitSound;
                         End;

                     End;

     tmdDownLeft : if MolXPos <> 0 Then
                     Begin
                       tmpGraphic := fGraphicArray.GetGraphic(MolXPos-1, MolYPos);

                       if (tmpGraphic is tMolStaticSprite) Then
                         if (tmpGraphic As tMolStaticSprite).IsCoin Then
                         Begin
                           inc (fNumCoins);
                           fMaze.InGameDestroyObject(fCurLocX + MolXPos-1, fCurLocY+MolYPos);
                           fGraphicArray.Remove_Graphic(tmpGraphic);
                           fMolSoundHandler.PlayGetCoinSound;
                         End;

                       if (tmpGraphic is tMolAnimatedSprite) Then
                         if (tmpGraphic As tMolAnimatedSprite).IsCircuit Then
                         Begin
                           inc (fNumCircuits);
                           fMaze.InGameDestroyObject(fCurLocX + MolXPos-1, fCurLocY+MolYPos);
                           fGraphicArray.Remove_Graphic(tmpGraphic);
                           fMolSoundHandler.PlayGetCircuitSound;
                         End;

                     End;
     end;
   end;
end;

// draws the panel showing time remaining, circuits collected etc.
procedure TFrmMolMan.DrawPanel (bmpDest : TBitMap);
Begin
  fPanelSetup.DrawLines(bmpDest.Canvas);
  fPanelSetup.DrawGraphics(bmpDest.Canvas);
  fPanelSetup.DrawTimeText(fTimeRemaining, bmpDest.Canvas);
  fPanelSetup.DrawBombText(fNumBombs, bmpDest.Canvas);
  fPanelSetup.DrawCoinsText(fNumCoins, bmpDest.Canvas);
  fPanelSetup.DrawCircuitsText(fNumCircuits, bmpDest.Canvas);
  fPanelSetup.DrawPillsText(fNumPills, bmpDest.Canvas);
End;

procedure TfrmMolMan.actHelpAboutExecute(Sender: TObject);
var
  frmAboutMolMan: TfrmAboutMolMan;
  bUnPause      : Boolean;

begin

  bUnPause := FALSE;
  
  // pause the game if necessary
  if not actNewGame.Enabled Then
    if not fPaused Then
    Begin
      actPauseGameExecute (Self);
      bUnPause := TRUE;
    End;

  frmAboutMolMan := TFrmAboutMolMan.Create (Self);
  frmAboutMolMan.SetVersionNo(VERSION);
  frmAboutMolMan.ShowModal;
  frmAboutMolMan.Free;

  // restart the game if necessary
  if bUnPause Then
    actPauseGameExecute (Self);
end;

procedure TfrmMolMan.actHelpInstructionsExecute(Sender: TObject);
var
  frmMolInstructions: TfrmMolInstructions;
  bUnPause      : Boolean;

begin

  bUnPause := FALSE;
  
  // pause the game if necessary
  if not actNewGame.Enabled Then
    if not fPaused Then
    Begin
      actPauseGameExecute (Self);
      bUnPause := TRUE;
    End;

  frmMolInstructions := TFrmMolInstructions.Create (Self);
  frmMolInstructions.ShowModal;
  frmMolInstructions.Free;

  // restart the game if necessary
  if bUnPause Then
    actPauseGameExecute (Self);

end;

procedure TfrmMolMan.actLoadMazeExecute(Sender: TObject);
begin
  if OPenDialog1.Execute Then
    fMaze.LoadMaze(OpenDialog1.FileName); 
end;

procedure TfrmMolMan.actRedfineKeysExecute(Sender: TObject);
var
  frmRedefineKeys : tFrmRedefineKeys;
  bUnPause        : Boolean;
begin

  bUnPause := FALSE;
  
  // pause the game if necessary
  if not actNewGame.Enabled Then
    if not fPaused Then
    Begin
      actPauseGameExecute (Self);
      bUnPause := TRUE;
    End;

  frmRedefineKeys := tFrmRedefineKeys.Create(Self);
  frmRedefineKeys.GameSetup := fGameSetup;
  frmRedefineKeys.Initialise;
  frmRedefineKeys.ShowModal;
  frmRedefineKeys.Free;

  // restart the game if necessary
  if bUnPause Then
    actPauseGameExecute (Self);
  
end;

procedure TfrmMolMan.tmrAmbientSoundsTimer(Sender: TObject);
begin
  fMolSoundHandler.PlayAmbientSound;

  // reset the timer interval to a new random value
  With fMolSoundHandler Do
    tmrAmbientSounds.Interval := (Random (MaxAmDelay - MinAmDelay) + MinAmDelay)*1000;

end;

initialization

  frmSplash := tFrmSplash.Create(Application);
  frmSplash.Show;
    While frmSplash.Visible do
      Application.ProcessMessages;

end.

