unit umolsoundhandler;

interface

{$IFDEF LINUX}
uses
  usharedmem;
{$ENDIF}

type
  tMolSoundHandler = class
  private

    fClock1 : String;
    fClock2 : String;
    fWalk   : String;
    fGetCoin: String;
    fGetCircuit : String;
    fNoMove : String;
    fBuy    : String;
    fBomb   : String;
    fDeath  : String;
    fCompleted : String;
    
    {$IFDEF LINUX}
    // sound identifiers and channel numbers for the sound player prog
    fClock1Num     : Integer;
    fClock1Chn     : Integer;
    fClock2Num     : Integer;
    fClock2Chn     : Integer;
    fWalkNum       : Integer;
    fWalkChn       : Integer;
    fGetCoinNum    : Integer;
    fGetCoinChn    : Integer;
    fGetCircuitNum : Integer;
    fGetCircuitChn : Integer;
    fNoMoveNum     : Integer;
    fNoMoveChn     : Integer;
    fBuyNum        : Integer;
    fBuyChn        : Integer;
    fBombNum       : Integer;             
    fBombChn       : Integer;             
    fDeathNum      : Integer;
    fDeathChn      : Integer;
    fCompletedNum  : Integer;
    fCompletedChn  : Integer;
    fAmbientNum    : Integer;
    fAmbientChn    : Integer;
    fpSoundRec     : pSoundRec;
    {$ENDIF}
    
    fNumAmbientSounds: Integer;
    fAmbientSounds   : Array of String;
    fMinAmDelay      : Integer;
    fMaxAmDelay      : Integer;
    fMagicNum        : Integer;
    fOldMagicNum     : Integer;
    fCurNumSounds    : Integer;
    
    procedure PlaySound (Filename : String); overload;
    procedure PlaySound (SoundNo, Channel : Integer); overload;

    {$IFDEF WIN32}
    procedure SendMCICommand(Cmd : string);
    {$ENDIF}
    {$IFDEF LINUX}
    procedure ClearSoundRec;
    {$ENDIF}
    
  public

    property  MinAmDelay : Integer Read fMinAmDelay;
    property  MaxAmDelay : Integer Read fMaxAmDelay;
    property  MagicNum   : Integer Read fMagicNum Write fMagicNum;

    procedure OpenAudio;
    procedure CloseAudio;
    procedure ReadMolSounds (IniFilename : String);
    procedure PlayWalkSound;
    procedure PlayClockSound (SndToPlay : Integer);
    procedure PlayGetCoinSound;
    procedure PlayGetCircuitSound;
    procedure PlayNoMoveSound;
    procedure PlayBombSound;
    procedure PlayBuySound;
    procedure PlayDeathSound;
    procedure PlayAmbientSound;
    procedure PlayCompletedSound;
  end;

implementation

uses
  SysUtils, IniFiles, QForms,uCompat,
  {$IFDEF LINUX}
  Libc;
  {$ENDIF}
  {$IFDEF WIN32}
  MMSystem;
  {$ENDIF}

{ tMolSoundHandler }

procedure tMolSoundHandler.PlayClockSound(SndToPlay: Integer);
begin
   // plays one or other of the clock ticking sounds, depending
   // on the value of sndtoplay
   {$IFDEF WIN32}
   if (SndToPlay Mod 2)= 0 Then
     PlaySound (fClock1)
   else
     PlaySound (fClock2);
   {$ELSE}
   if (SndToPlay Mod 2)= 0 Then
     PlaySound (fClock1Num, fClock1Chn)
   else
     PlaySound (fClock2Num, fClock2Chn);
   {$ENDIF}
end;

{$IFDEF LINUX}
// sets the sound record to indicate no sounds are to be played
procedure tMolSoundHandler.ClearSoundRec;
var
  iLoop : Integer;
Begin
  For iLoop := 1 to MAX_NUM_SOUNDS do
    fpSoundRec.SoundInfo[iLoop].SoundNo := NO_SOUND;
End;
{$ENDIF}

procedure tMolSoundHandler.PlaySound(SoundNo, Channel :Integer);
Begin
  {$IFDEF LINUX}
  // are we starting a new set of sounds
  if fOldMagicNum <> fMagicNum Then
  Begin
    fpSoundRec^.MagicNo := MagicNum;
    fOldMagicNum := MagicNum;
    fCurNumSounds := 1;
    ClearSoundRec;
  End;

  // write the sound and channel number to be played to shared memory
  // the sound player will read this info and play the sound
  if fCurNumSounds <= MAX_NUM_SOUNDS Then
  Begin
    fpSoundRec.SoundInfo [fCurNumSounds].SoundNo := SoundNo;
    fpSoundRec.SoundInfo [fCurNumSounds].SoundChn := Channel;
    inc (fCurNumSounds);
  End;
  {$ENDIF}
End;

procedure tMolSoundHandler.PlaySound(Filename: String);
{$IFDEF WIN32}
var
  sTemp : String;
{$ENDIF}

Begin
   if Filename ='' Then exit;

  {$IFDEF LINUX}
    // no longer used - did call an external player, but was extremely laggy
    //    sTemp := fSoundPlayer +' '+RespecFilename (ExtractFilePath (Application.Exename)+
    //            sFilename) +' > /dev/null &';
    //    Libc.system(PChar (sTemp));
  {$ENDIF}
  {$IFDEF WIN32}
    sTemp := RespecFilename (ExtractFilePath (Application.Exename)+Filename);
    SendMCICommand('play '+sTemp);
  {$ENDIF}

end;

procedure tMolSoundHandler.PlayWalkSound;
begin
  {$IFDEF LINUX}
  PlaySound (fWalkNum, fWalkChn);
  {$ELSE}
  Playsound (fWalk);
  {$ENDIF}
end;

procedure tMolSoundHandler.ReadMolSounds(IniFilename: String);
var
  IniSounds : TIniFile;
  iLoop     : Integer;

begin
  IniSounds := TInifile.Create(IniFilename);

  fWalk := IniSounds.ReadString('General', 'walk', '');
  fClock1 := IniSounds.ReadString ('General', 'clocksound1', '');
  fClock2 := IniSounds.ReadString ('General', 'clocksound2', '');
  fGetCoin := IniSounds.ReadString('General','getcoin','');
  fGetCircuit := IniSounds.ReadString('General','getcircuit','');
  fNoMove := IniSounds.ReadString('General','cantmove','');
  fBuy := IniSounds.ReadString('General','buy','');
  fBomb := IniSounds.ReadString('General','bomb','');
  fDeath := IniSounds.ReadString('General','death','');
  fCompleted := IniSounds.ReadString('General','completed','');

  {$IFDEF LINUX}
  fClock1Num     := IniSounds.ReadInteger('General','clocksound1num',0); 
  fClock1Chn     := IniSounds.ReadInteger('General','clocksound1chn',0); 
  fClock2Num     := IniSounds.ReadInteger('General','clocksound2num',0); 
  fClock2Chn     := IniSounds.ReadInteger('General','clocksound2chn',0); 
  fWalkNum       := IniSounds.ReadInteger('General','walknum',0); 
  fWalkChn       := IniSounds.ReadInteger('General','walkchn',0); 
  fGetCoinNum    := IniSounds.ReadInteger('General','getcoinnum',0); 
  fGetCoinChn    := IniSounds.ReadInteger('General','getcoinchn',0); 
  fGetCircuitNum := IniSounds.ReadInteger('General','getcircuitnum',0); 
  fGetCircuitChn := IniSounds.ReadInteger('General','getcircuitchn',0); 
  fNoMoveNum     := IniSounds.ReadInteger('General','cantmovenum',0); 
  fNoMoveChn     := IniSounds.ReadInteger('General','cantmovechn',0); 
  fBuyNum        := IniSounds.ReadInteger('General','buynum',0); 
  fBuyChn        := IniSounds.ReadInteger('General','buychn',0); 
  fBombNum       := IniSounds.ReadInteger('General','bombnum',0); 
  fBombChn       := IniSounds.ReadInteger('General','bombchn',0); 
  fDeathNum      := IniSounds.ReadInteger('General','deathnum',0); 
  fDeathChn      := IniSounds.ReadInteger('General','deathchn',0); 
  fCompletedNum  := IniSounds.ReadInteger('General','completednum',0); 
  fCompletedChn  := IniSounds.ReadInteger('General','completedchn',0); 
  fAmbientNum    := IniSounds.ReadInteger('AmbientSounds','ambientnum',0); 
  fAmbientChn    := IniSounds.ReadInteger('AmbientSounds','ambientchn',0); 
  {$ENDIF}

  // read in the ambient sound details
  fNumAmbientSounds := IniSounds.ReadInteger('AmbientSounds','numsounds', 0);
  fMinAmDelay := IniSounds.ReadInteger('AmbientSounds', 'minsecsbetweensounds', 30);
  fMaxAmDelay := IniSounds.ReadInteger('AmbientSounds', 'minsecsbetweensounds', 120);
  SetLength (fAmbientSounds, fNumAmbientSounds);
  For iLoop := 1 to fNumAmbientSounds do
    fAmbientSounds[iLoop-1] := IniSounds.ReadString
                               ('AmbientSounds', 'sound'+IntToStr (iLoop),'');

  IniSounds.Free;

  {$IFDEF LINUX}
  // create a shared memory area and start the sound player
  fpSoundRec := CreateSharedMem;
  ClearSoundRec;
  Libc.system(PChar (ExtractFilePath (Application.Exename) + '/molsoundplayer &')); 
  fMagicNum := -1;
  fOldMagicNum :=1;
  fCurNumSounds :=1;
  {$ENDIF}
end;

{$IFDEF WIN32}

procedure TMolSoundHandler.SendMCICommand(Cmd : string);
var
  RetVal : integer;
begin
  RetVal := mciSendString(Pchar (Cmd), nil, 0, 0);
  if RetVal <> 0 then;
end;
{$ENDIF}

procedure tMolSoundHandler.CloseAudio;
begin
  {$IFDEF WIN32}
  SendMCICommand('open waveaudio shareable');
  {$ENDIF}
end;

procedure tMolSoundHandler.OpenAudio;
begin
  {$IFDEF WIN32}
  SendMCICommand('close waveaudio');
  {$ENDIF}
end;

procedure tMolSoundHandler.PlayGetCoinSound;
begin
  {$IFDEF WIN32}
  Playsound (fGetCoin);
  {$ELSE}
  Playsound (fGetCoinNum, fGetCoinChn);
  {$ENDIF}
end;

procedure tMolSoundHandler.PlayNoMoveSound;
begin
  {$IFDEF WIN32}
  Playsound (fNoMove);
  {$ELSE}
  Playsound (fNoMoveNum, fNoMoveChn);
  {$ENDIF}
end;

procedure tMolSoundHandler.PlayBombSound;
begin
  {$IFDEF WIN32}
  PlaySound (fBomb);
  {$ELSE}
  PlaySound (fBombNum, fBombChn);
  {$ENDIF}
end;

procedure tMolSoundHandler.PlayBuySound;
begin
  {$IFDEF WIN32}
  PlaySound (fBuy);
  {$ELSE}
  PLaySound (fBuyNum, fBuyChn);
  {$ENDIF}
end;

procedure tMolSoundHandler.PlayDeathSound;
begin
  {$IFDEF WIN32}
  Playsound (fDeath);
  {$ELSE}
  Playsound (fDeathNum, fDeathChn);
  {$ENDIF}
end;

procedure tMolSoundHandler.PlayAmbientSound;
begin
  {$IFDEF WIN32}
  // play a random ambient sound
  PlaySound (fAmbientSounds [Random (fNumAmbientSounds)]);
  {$ELSE}
  // the sound player will pick which ambient sound to play
  PLaySound (fAmbientNum, fAmbientChn);
  {$ENDIF}
end;

procedure tMolSoundHandler.PlayCompletedSound;
begin
  {$IFDEF WIN32}
  Playsound (fCompleted);
  {$ELSE}
  Playsound (fCompletedNum,fCompletedChn);
  {$ENDIF}
end;

procedure tMolSoundHandler.PlayGetCircuitSound;
begin
  {$IFDEF WIN32}
  PlaySound (fGetCircuit);
  {$ELSE}
  PlaySound (fGetCircuitNum, fGetCircuitChn);
  {$ENDIF}
end;

end.
