unit usharedmem;

interface

uses
  Libc;

const
  NO_SOUND =-1 ;
  MAX_NUM_SOUNDS = 8;
    
type
  tSoundInfo = 
  Record
    SoundNo  : Integer;
    SoundChn : Integer;
  End;

  pSoundRec = ^tSoundRec;  
  tSoundRec = 
  Record
     MagicNo : Integer;        // a change in this indicates new sound info
     SoundInfo : Array[1 .. MAX_NUM_SOUNDS] of tSoundInfo;
  End;
  
function  CreateSharedMem  : pointer;
function  GetSharedMem : pointer;
procedure DeleteSharedMem;
procedure DetatchSharedMem (ptrToMem : Pointer);

implementation

var
  shmid : Integer;
  
const
  MEM_KEY=123456;

function GetSharedMem : pointer;
begin

  Result := NIL;
  
  // gets the shmid of an already created shared memory area and
  // uses it to attach the memory into the memory map
  shmid := shmget (MEM_KEY, SizeOf (tSoundRec) , 438);
  if shmid = -1 Then
  Begin
    writeln ('Get shared mem failed - err no is ', Libc.errno);
    Exit;
  End;
  
  Result := shmat (shmid, NIL, 0);
end;
  
// creates an area of shared memory big enough to hold a tSoundRec, attaches 
// it to the apps memory space and returns a pointer to it 
function CreateSharedMem  : pointer;
Begin
  Result := NIL;
  shmid := shmget (MEM_KEY, sizeof (tSoundRec), 438 or IPC_CREAT);
  if shmid = -1 Then
  Begin
    shmid := Libc.errno;
    Writeln ('Failure in create shared mem - ', shmid);
    exit;  // error
  End;
  // now attach it  
  Result := shmat (shmid, NIL, 0);
End;

procedure DetatchSharedMem (ptrToMem : Pointer);
begin
  shmdt (ptrToMem);
end;

procedure DeleteSharedMem;
begin
  shmctl (shmid, IPC_RMID, nil);
end;  
  
end.
 