unit usplashscreen;

interface

uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, QExtCtrls, QComCtrls,  uMolGraphics;

type
  Tfrmsplash = class(TForm)
    Image1: TImage;
    ProgressBar1: TProgressBar;
    Timer1: TTimer;
    procedure Timer1Timer(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    fGameGraphics : tGameGraphicsArray;
    fMolManGraphics : tMolManGraphicsArray;
    { Private declarations }
  public
     property GameGraphics : tGameGraphicsArray  Read fGameGraphics;
     property MolManGraphics : tMolManGraphicsArray Read fMolManGraphics;

    { Public declarations }
  end;

implementation

uses
  inifiles, uCompat;

{$R *.xfm}

procedure Tfrmsplash.Timer1Timer(Sender: TObject);
var
  iLoop     : Integer;
  tmpBitmap : tBitmap;
  sSection  : String;
  IniGraph  : TInifile;
  iNumGraphics : Integer;
  sFilename : String;
  iNumFrames: Integer;
  iLoop1    : Integer;

begin

  // load all the graphics into memory, so that no file access is needed
  // during the game
  Timer1.Enabled := FALSE;

  IniGraph := TIniFile.Create (RespecFilename (ExtractFilePath (Application.ExeName) +'/molgraphics.ini'));

  iNumGraphics := IniGraph.ReadInteger('General','numgraphics', 0);
  SetLength (fGameGraphics, iNUmGraphics+1);

  ProgressBar1.Max := iNumGraphics+1;

  For iLoop := 0 to iNumGraphics do
  Begin

    sSection := 'MolGraphic'+IntToStr (iLoop);
    if IniGraph.ReadBool (sSection, 'moleculeman', FALSE) Then
    Begin
      SetLength (fGameGraphics[iLoop], 1);
      fGameGraphics [iLoop,0] := NIL;

      //molecule man - faces four direction of NUM_PLAYER_ANIMS frames
      For iLoop1 := 1 to NUM_PLAYER_ANIMS do
      Begin
        tmpBitMap := tBItmap.Create;
        tmpBitmap.LoadFromFile (ReSpecFilename (IniGraph.ReadString
                               (sSection, 'upleft'+intToStr(iLoop1),'')));
        fMolManGraphics [tmdUpLeft, iLoop1] := tmpBitmap;
      End;

      For iLoop1 := 1 to NUM_PLAYER_ANIMS do
      Begin
        tmpBitMap := tBItmap.Create;
        tmpBitmap.LoadFromFile (RespecFilename (IniGraph.ReadString
                               (sSection, 'upright'+intToStr(iLoop1),'')));
        fMolManGraphics [tmdUpRight, iLoop1] := tmpBitmap;
      End;

      For iLoop1 := 1 to NUM_PLAYER_ANIMS do
      Begin
        tmpBitMap := tBItmap.Create;
        tmpBitmap.LoadFromFile (RespecFilename (IniGraph.ReadString
                               (sSection, 'downright'+intToStr(iLoop1),'')));
        fMolManGraphics [tmdDownRight, iLoop1] := tmpBitmap;
      End;

      For iLoop1 := 1 to NUM_PLAYER_ANIMS do
      Begin
        tmpBitMap := tBItmap.Create;
        tmpBitmap.LoadFromFile (Respecfilename (IniGraph.ReadString
                                (sSection, 'downleft'+intToStr(iLoop1),'')));
        fMolManGraphics [tmdDownleft, iLoop1] := tmpBitmap;
      End;

    End
    Else
      if IniGraph.ReadBool (sSection, 'animated', FALSE) Then
      Begin
        // animated graphic - need to read all of the frames
        iNumFrames := IniGraph.ReadInteger (sSection, 'numframes', 1);
        SetLength (fGameGraphics[iLoop], iNumFrames);
        for iLoop1 := 0 to iNumFrames-1 do
        Begin
          tmpBitMap := tBItmap.Create;
          sFileName := IniGraph.ReadString (sSection, 'graphic'+inttostr (iLoop1+1),'');
          sFilename := RespecFilename (sFilename);
          tmpBitmap.LoadFromFile (sFileName);
          fGameGraphics [iLoop,iLoop1] := tmpBitmap;
        End
      End
        else
        Begin
          SetLength (fGameGraphics[iLoop], 1);
          sFilename := IniGraph.ReadString (sSection, 'graphic', '');
          sFilename := RespecFilename (sFilename);
          tmpBitMap := tBItmap.Create;
          tmpBitmap.LoadFromFile (sFileName);
          fGameGraphics [iLoop,0] := tmpBitmap;
        End;
    try
      ProgressBar1.StepIt;
      Application.ProcessMessages;
    except
      MessageDlg ('Fatal Error', 'Could not load graphics file '+ sFileName+#13#10+
                  'Molecule Man will now shut down.', mtError, [mbOk], 0);
      Close;
    end;
  End;

  Hide;
end;

procedure Tfrmsplash.FormDestroy(Sender: TObject);
var
  iLoop  : Integer;
  iLoop1 : Integer;
  iLoop2 : tMovementDir;

begin
  // free the graphics we loaded earlier
  for iLoop := 0 to Length (fGameGraphics) -1 do
    for iLoop1 := 0 to Length (fGameGraphics[iLoop]) -1 do
      if Assigned (fGameGRaphics [iLoop, iLoop1]) then
        fGameGraphics [iLoop, iLoop1].Free;

  for iLoop2 :=  tmdUpLeft to tmdDownLeft do
    for iLoop1 := 1 to NUM_PLAYER_ANIMS do
       fMolManGraphics[iLoop2, iLoop1].Free;
end;

end.
